@@ -8,7 +8,7 @@
 # Copyright (c) 1998-2000 by Ajuba Solutions.
 # All rights reserved.
 # 
-# RCS: @(#) $Id: all.tcl,v 1.13 2003/05/02 07:42:05 andreas_kupries Exp $
+# RCS: @(#) $Id$
 
 set old_auto_path $auto_path
 
@@ -145,6 +145,12 @@ foreach module $modules {
 	set ::argv0 [pSet ::argv0]
 	set ::tcllibModule [pSet module]
 	set auto_path [pSet auto_path]
+
+	# The next command allows the execution of 'tk' constrained
+	# tests, if Tk is present (for example when this code is run
+	# run by 'wish').
+	catch {package require Tk}
+
 	package require tcltest
 	namespace import ::tcltest::*
 	set ::tcltest::testSingleFile false
@@ -155,9 +161,9 @@ foreach module $modules {
 	# 'wrong#args' situations. The format of the messages changed
 	# for 8.4
 
-	proc ::tcltest::getErrorMessage {functionName argList missingIndex} {
+	proc ::tcltest::wrongNumArgs {functionName argList missingIndex} {
 	    # if oldstyle errors:
-	    if { [info tclversion] < 8.4 } {
+	    if {[package vcompare [package provide Tcl] 8.4] < 0} {
 		set msg "no value given for parameter "
 		append msg "\"[lindex $argList $missingIndex]\" to "
 		append msg "\"$functionName\""
@@ -167,15 +173,34 @@ foreach module $modules {
 	    return $msg
 	}
 
-	proc ::tcltest::tooManyMessage {functionName argList} {
+	proc ::tcltest::tooManyArgs {functionName argList} {
 	    # if oldstyle errors:
-	    if { [info tclversion] < 8.4 } {
+	    if {[package vcompare [package provide Tcl] 8.4] < 0} {
 		set msg "called \"$functionName\" with too many arguments"
 	    } else {
 		set msg "wrong # args: should be \"$functionName $argList\""
 	    }
 	    return $msg
 	}
+
+	# This constraint restricts certain tests to run on tcl 8.3+, and tcl8.4+
+	if {[package vsatisfies [package provide tcltest] 2.0]} {
+	    # tcltest2.0+ has an API to specify a test constraint
+	    ::tcltest::testConstraint tcl8.3only \
+		    [expr {![package vsatisfies [package provide Tcl] 8.4]}]
+	    ::tcltest::testConstraint tcl8.3plus \
+		    [expr {[package vsatisfies [package provide Tcl] 8.3]}]
+	    ::tcltest::testConstraint tcl8.4plus \
+		    [expr {[package vsatisfies [package provide Tcl] 8.4]}]
+	} else {
+	    # In tcltest1.0, a global variable needs to be set directly.
+	    set ::tcltest::testConstraints(tcl8.3only) \
+		    [expr {![package vsatisfies [package provide Tcl] 8.4]}]
+	    set ::tcltest::testConstraints(tcl8.3plus) \
+		    [expr {[package vsatisfies [package provide Tcl] 8.3]}]
+	    set ::tcltest::testConstraints(tcl8.4plus) \
+		    [expr {[package vsatisfies [package provide Tcl] 8.4]}]
+	}
     }
     interp alias $c ::tcltest::cleanupTestsHook {} \
 	    ::tcltest::cleanupTestsHook $c
