@@ -7,9 +7,11 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 #
-# RCS: @(#) $Id: counter.tcl,v 1.6 2001/07/10 20:39:46 andreas_kupries Exp $
+# RCS: @(#) $Id$
 
-namespace eval counter:: {
+package require Tcl 8.2
+
+namespace eval ::counter {
 
     # Variables of name counter::T-$tagname
     # are created as arrays to support each counter.
@@ -31,15 +33,16 @@ namespace eval counter:: {
     # of counters to do mergeing on.
 
     variable tagsToMerge
-    if {![info exist tagsToMerge]} {
+    if {![info exists tagsToMerge]} {
 	set tagsToMerge {}
     }
     variable mergeInterval
 
-    namespace export *
+    namespace export init reset count exists get names start stop
+    namespace export histHtmlDisplay histHtmlDisplayRow histHtmlDisplayBarChart
 }
 
-# counter::init --
+# ::counter::init --
 #
 #	Set up a counter.
 #
@@ -54,7 +57,7 @@ namespace eval counter:: {
 # Side Effects:
 #	Initializes state about a counter.
 
-proc counter::init {tag args} {
+proc ::counter::init {tag args} {
     upvar #0 counter::T-$tag counter
     if {[info exists counter]} {
 	unset counter
@@ -92,7 +95,7 @@ proc counter::init {tag args} {
 		for {set i 0} {$i < 24} {incr i} {
 		    set hourhist($i) 0
 		}
-		if {[info exist dayhist]} {
+		if {[info exists dayhist]} {
 		    unset dayhist
 		}
 		set dayhist(0) 0
@@ -118,7 +121,7 @@ proc counter::init {tag args} {
 		# and starting times to keep them all synchronized.
 		# So, we only initialize these parameters once.
 
-		if {![info exist secsPerMinute]} {
+		if {![info exists secsPerMinute]} {
 		    set secsPerMinute $value
 
 		    set startTime [clock seconds]
@@ -167,7 +170,7 @@ proc counter::init {tag args} {
 		# Cluster a set of counters with a single total
 
 		upvar #0 counter::H-$tag histogram
-		if {[info exist histogram]} {
+		if {[info exists histogram]} {
 		    unset histogram
 		}
 		set counter(group) $value
@@ -178,7 +181,7 @@ proc counter::init {tag args} {
 		upvar #0 counter::V-$tag vector
 		set counter(lastn) $value
 		set counter(index) 0
-		if {[info exist vector]} {
+		if {[info exists vector]} {
 		    unset vector
 		}
 		for {set i 0} {$i < $value} {incr i} {
@@ -189,7 +192,7 @@ proc counter::init {tag args} {
 		# A value-based histogram with buckets for different values.
 
 		upvar #0 counter::H-$tag histogram
-		if {[info exist histogram]} {
+		if {[info exists histogram]} {
 		    unset histogram
 		}
 		set counter(bucketsize) $value
@@ -197,7 +200,7 @@ proc counter::init {tag args} {
 	    }
 	    -hist2x {
 		upvar #0 counter::H-$tag histogram
-		if {[info exist histogram]} {
+		if {[info exists histogram]} {
 		    unset histogram
 		}
 		set counter(bucketsize) $value
@@ -205,7 +208,7 @@ proc counter::init {tag args} {
 	    }
 	    -hist10x {
 		upvar #0 counter::H-$tag histogram
-		if {[info exist histogram]} {
+		if {[info exists histogram]} {
 		    unset histogram
 		}
 		set counter(bucketsize) $value
@@ -213,7 +216,7 @@ proc counter::init {tag args} {
 	    }
 	    -histlog {
 		upvar #0 counter::H-$tag histogram
-		if {[info exist histogram]} {
+		if {[info exists histogram]} {
 		    unset histogram
 		}
 		set counter(bucketsize) $value
@@ -244,7 +247,7 @@ proc counter::init {tag args} {
     return ""
 }
 
-# counter::reset --
+# ::counter::reset --
 #
 #	Reset a counter.
 #
@@ -257,7 +260,7 @@ proc counter::init {tag args} {
 # Side Effects:
 #	Deletes the counter and calls counter::init again for it.
 
-proc counter::reset {tag args} {
+proc ::counter::reset {tag args} {
     upvar #0 counter::T-$tag counter
 
     # Layer reset on top of init.  Here we figure out what
@@ -269,14 +272,14 @@ proc counter::reset {tag args} {
 	}
 	-group {
 	    upvar #0 counter::H-$tag histogram
-	    if {[info exist histogram]} {
+	    if {[info exists histogram]} {
 		unset histogram
 	    }
 	    set args [list -group $counter(group)]
 	}
 	-lastn {
 	    upvar #0 counter::V-$tag vector
-	    if {[info exist vector]} {
+	    if {[info exists vector]} {
 		unset vector
 	    }
 	    set args [list -lastn $counter(lastn)]
@@ -286,7 +289,7 @@ proc counter::reset {tag args} {
 	-histlog -
 	-hist2x {
 	    upvar #0 counter::H-$tag histogram
-	    if {[info exist histogram]} {
+	    if {[info exists histogram]} {
 		unset histogram
 	    }
 	    set args [list $counter(type) $counter(bucketsize)]
@@ -294,7 +297,7 @@ proc counter::reset {tag args} {
 	-timehist {
 	    foreach h [list counter::H-$tag counter::Hour-$tag counter::Day-$tag] {
 		upvar #0 $h histogram
-		if {[info exist histogram]} {
+		if {[info exists histogram]} {
 		    unset histogram
 		}
 	    }
@@ -308,7 +311,7 @@ proc counter::reset {tag args} {
     return ""
 }
 
-# counter::count --
+# ::counter::count --
 #
 #	Accumulate statistics.
 #
@@ -323,7 +326,7 @@ proc counter::reset {tag args} {
 # Side Effects:
 #	Accumlate statistics.
 
-proc counter::count {tag {delta 1} args} {
+proc ::counter::count {tag {delta 1} args} {
     upvar #0 counter::T-$tag counter
     set counter(total) [expr {$counter(total) + $delta}]
     incr counter(N)
@@ -353,7 +356,7 @@ proc counter::count {tag {delta 1} args} {
 	    -hist {
 		upvar #0 counter::H-$tag histogram
 		set bucket [expr {int($delta / $counter(bucketsize))}]
-		if {![info exist histogram($bucket)]} {
+		if {![info exists histogram($bucket)]} {
 		    set histogram($bucket) 0
 		}
 		incr histogram($bucket)
@@ -366,7 +369,7 @@ proc counter::count {tag {delta 1} args} {
 			{set max [expr {$max * $counter(mult)}]} {
 		    incr bucket
 		}
-		if {![info exist histogram($bucket)]} {
+		if {![info exists histogram($bucket)]} {
 		    set histogram($bucket) 0
 		}
 		incr histogram($bucket)
@@ -374,7 +377,7 @@ proc counter::count {tag {delta 1} args} {
 	    -histlog {
 		upvar #0 counter::H-$tag histogram
 		set bucket [expr {int(log($delta)*$counter(bucketsize))}]
-		if {![info exist histogram($bucket)]} {
+		if {![info exists histogram($bucket)]} {
 		    set histogram($bucket) 0
 		}
 		incr histogram($bucket)
@@ -411,7 +414,7 @@ proc counter::count {tag {delta 1} args} {
     return
 }
 
-# counter::exists --
+# ::counter::exists --
 #
 #	Return true if the counter exists.
 #
@@ -424,12 +427,12 @@ proc counter::count {tag {delta 1} args} {
 # Side Effects:
 #	None.
 
-proc counter::exists {tag} {
+proc ::counter::exists {tag} {
     upvar #0 counter::T-$tag counter
     return [info exists counter]
 }
 
-# counter::get --
+# ::counter::get --
 #
 #	Return statistics.
 #
@@ -444,7 +447,7 @@ proc counter::exists {tag} {
 # Side Effects:
 #	None.
 
-proc counter::get {tag {option -total} args} {
+proc ::counter::get {tag {option -total} args} {
     upvar #0 counter::T-$tag counter
     switch -- $option {
 	-total {
@@ -480,7 +483,7 @@ proc counter::get {tag {option -total} args} {
 	    if {[llength $args]} {
 		# Return particular bucket
 		set bucket [lindex $args 0]
-		if {[info exist histogram($bucket)]} {
+		if {[info exists histogram($bucket)]} {
 		    return $histogram($bucket)
 		} else {
 		    return 0
@@ -552,7 +555,7 @@ proc counter::get {tag {option -total} args} {
     }
 }
 
-# counter::names --
+# ::counter::names --
 #
 #	Return the list of defined counters.
 #
@@ -565,19 +568,19 @@ proc counter::get {tag {option -total} args} {
 # Side Effects:
 #	None.
 
-proc counter::names {} {
+proc ::counter::names {} {
     set result {}
     foreach v [info vars ::counter::T-*] {
-	if {[info exist $v]} {
+	if {[info exists $v]} {
 	    # Declared arrays might not exist, yet
-	    regsub -- ::counter::T- $v {} v
+	    set v [string map {{::counter::T-} {}} $v]
 	    lappend result $v
 	}
     }
     return $result
 }
 
-# counter::MergeHour --
+# ::counter::MergeHour --
 #
 #	Sum the per-minute histogram into the next hourly bucket.
 #	On 24-hour boundaries, sum the hourly buckets into the next day bucket.
@@ -592,7 +595,7 @@ proc counter::names {} {
 # Side Effects:
 #	See description.
 
-proc counter::MergeHour {interval} {
+proc ::counter::MergeHour {interval} {
     variable hourIndex
     variable minuteBase
     variable hourBase
@@ -600,7 +603,7 @@ proc counter::MergeHour {interval} {
     variable secsPerMinute
 
     after $interval [list counter::MergeHour $interval]
-    if {![info exist hourBase] || $hourIndex == 0} {
+    if {![info exists hourBase] || $hourIndex == 0} {
 	set hourBase $minuteBase
     }
     set minuteBase [clock seconds]
@@ -638,7 +641,7 @@ proc counter::MergeHour {interval} {
     }
 
 }
-# counter::MergeDay --
+# ::counter::MergeDay --
 #
 #	Sum the per-minute histogram into the next hourly bucket.
 #	On 24-hour boundaries, sum the hourly buckets into the next day bucket.
@@ -653,7 +656,7 @@ proc counter::MergeHour {interval} {
 # Side Effects:
 #	See description.
 
-proc counter::MergeDay {} {
+proc ::counter::MergeDay {} {
     variable dayIndex
     variable dayBase
     variable hourBase
@@ -663,7 +666,7 @@ proc counter::MergeDay {} {
     # Save the hours histogram into a bucket for the last day
     # counter(day,$day) is the starting time for that day bucket
 
-    if {![info exist dayBase]} {
+    if {![info exists dayBase]} {
 	set dayBase $hourBase
     }
     foreach tag $tagsToMerge {
@@ -673,7 +676,7 @@ proc counter::MergeDay {} {
 	set dayhist($dayIndex) 0
 	set max 0
 	for {set i 0} {$i < 24} {incr i} {
-	    if {[info exist hourhist($i)]} {
+	    if {[info exists hourhist($i)]} {
 		set dayhist($dayIndex) [expr {$dayhist($dayIndex) + $hourhist($i)}]
 		if {$hourhist($i) > $max} { 
 		    set mx $hourhist($i) 
@@ -692,7 +695,7 @@ proc counter::MergeDay {} {
     incr dayIndex
 }
 
-# counter::histHtmlDisplay --
+# ::counter::histHtmlDisplay --
 #
 #	Create an html display of the histogram.
 #
@@ -719,14 +722,14 @@ proc counter::MergeDay {} {
 # Side Effects:
 #	None.
 
-proc counter::histHtmlDisplay {tag args} {
+proc ::counter::histHtmlDisplay {tag args} {
     append result "<p>\n<table border=0 cellpadding=0 cellspacing=0>\n"
     append result [eval {counter::histHtmlDisplayRow $tag} $args]
     append result </table>
     return $result
 }
 
-# counter::histHtmlDisplayRow --
+# ::counter::histHtmlDisplayRow --
 #
 #	Create an html display of the histogram.
 #
@@ -740,7 +743,7 @@ proc counter::histHtmlDisplay {tag args} {
 # Side Effects:
 #	None.
 
-proc counter::histHtmlDisplayRow {tag args} {
+proc ::counter::histHtmlDisplayRow {tag args} {
     upvar #0 counter::T-$tag counter
     variable secsPerMinute
     variable minuteBase
@@ -776,7 +779,7 @@ proc counter::histHtmlDisplayRow {tag args} {
 	min* {
 	    upvar #0 counter::H-$tag histogram
 	    set histname counter::H-$tag
-	    if {![info exist minuteBase]} {
+	    if {![info exists minuteBase]} {
 		return "<!-- No time-based histograms defined -->"
 	    }
 	    set time $minuteBase
@@ -789,7 +792,7 @@ proc counter::histHtmlDisplayRow {tag args} {
 	hour* {
 	    upvar #0 counter::Hour-$tag histogram
 	    set histname counter::Hour-$tag
-	    if {![info exist hourBase]} {
+	    if {![info exists hourBase]} {
 		return "<!-- Hour merge has not occurred -->"
 	    }
 	    set time $hourBase
@@ -805,7 +808,7 @@ proc counter::histHtmlDisplayRow {tag args} {
 	day* {
 	    upvar #0 counter::Day-$tag histogram
 	    set histname counter::Day-$tag
-	    if {![info exist dayBase]} {
+	    if {![info exists dayBase]} {
 		return "<!-- Hour merge has not occurred -->"
 	    }
 	    set time $dayBase
@@ -938,7 +941,7 @@ proc counter::histHtmlDisplayRow {tag args} {
 	if {$options(-max) < 0} {
 	    set options(-max) [lindex $ix end]
 	}
-	if {![info exist options(-min)]} {
+	if {![info exists options(-min)]} {
 	    set options(-min) [lindex $ix 0]
 	}
     }
@@ -966,7 +969,7 @@ proc counter::histHtmlDisplayRow {tag args} {
     return $result
 }
 
-# counter::histHtmlDisplayBarChart --
+# ::counter::histHtmlDisplayBarChart --
 #
 #	Create an html display of the histogram.
 #
@@ -984,7 +987,7 @@ proc counter::histHtmlDisplayRow {tag args} {
 # Side Effects:
 #	See description.
 
-proc counter::histHtmlDisplayBarChart {tag histVar max curIndex time args} {
+proc ::counter::histHtmlDisplayBarChart {tag histVar max curIndex time args} {
     upvar #0 counter::T-$tag counter
     upvar 1 $histVar histogram
     variable secsPerMinute
@@ -995,7 +998,7 @@ proc counter::histHtmlDisplayBarChart {tag histVar max curIndex time args} {
     set ix [lsort -integer [array names histogram]]
 
     for {set t $options(-min)} {$t < $options(-max)} {incr t} {
-	if {![info exist histogram($t)]} {
+	if {![info exists histogram($t)]} {
 	    set value 0
 	} else {
 	    set value $histogram($t)
@@ -1165,7 +1168,7 @@ proc counter::histHtmlDisplayBarChart {tag histVar max curIndex time args} {
     return $result
 }
 
-# counter::start --
+# ::counter::start --
 #
 #	Start an interval timer.  This should be pre-declared with
 #	type either -hist, -hist2x, or -hist20x
@@ -1181,13 +1184,16 @@ proc counter::histHtmlDisplayBarChart {tag histVar max curIndex time args} {
 # Side Effects:
 #	Records the starting time for the instance of this interval.
 
-proc counter::start {tag instance} {
+proc ::counter::start {tag instance} {
     upvar #0 counter::Time-$tag time
-    set time($instance) [list [clock clicks] \
-	    [clock seconds]]
+    # clock clicks can return negative values if the sign bit is set
+    # Here we turn it into a 31-bit counter because we only want
+    # relative differences
+    set msec [expr [clock clicks -milliseconds] & 0x7FFFFFFF]
+    set time($instance) [list $msec [clock seconds]]
 }
 
-# counter::stop --
+# ::counter::stop --
 #
 #	Record an interval timer.
 #
@@ -1204,30 +1210,36 @@ proc counter::start {tag instance} {
 # Side Effects:
 #	Computes the current interval and adds it to the histogram.
 
-proc counter::stop {tag instance {func ::counter::Identity}} {
+proc ::counter::stop {tag instance {func ::counter::Identity}} {
     upvar #0 counter::Time-$tag time
 
-    if {![info exist time($instance)]} {
+    if {![info exists time($instance)]} {
 	# Extra call. Ignore so we can debug error cases.
 	return
     }
-    set now [list [clock clicks] \
-	    [clock seconds]]
+    set msec [expr [clock clicks -milliseconds] & 0x7FFFFFFF]
+    set now [list $msec [clock seconds]]
     set delMicros [expr {[lindex $now 0] - [lindex $time($instance) 0]}]
+    if {$delMicros < 0} {
+      # Microsecond counter wrapped.
+      set delMicros [expr {0x7FFFFFFF - [lindex $time($instance) 0] +
+                            [lindex $now 0]}]
+    }
     set delSecond [expr {[lindex $now 1] - [lindex $time($instance) 1]}]
     unset time($instance)
 
-    if {$delMicros < 0} {
-	set delMicros [expr {1000000 + $delMicros}]
-	incr delSecond -1
-	if {$delSecond < 0} {
-	    set delSecond 0
-	}
+    # It is quite possible that the millisecond counter is much
+    # larger than 1000, so we just use it unless our microsecond
+    # calculation is screwed up.
+
+    if {$delMicros >= 0} {
+      counter::count $tag [$func [expr {$delMicros / 1000.0}]]
+    } else {
+      counter::count $tag [$func $delSecond]
     }
-    counter::count $tag [$func $delSecond.[format %06d $delMicros]]
 }
 
-# counter::Identity --
+# ::counter::Identity --
 #
 #	Return its argument.  This is used as the default function
 #	to apply to an interval timer.
@@ -1242,9 +1254,9 @@ proc counter::stop {tag instance {func ::counter::Identity}} {
 #	None
 
 
-proc counter::Identity {x} {
+proc ::counter::Identity {x} {
     return $x
 }
 
-package provide counter 2.0
+package provide counter 2.0.2
 
