@@ -8,11 +8,11 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id: fileutil.tcl,v 1.23 2003/05/01 22:40:14 patthoyts Exp $
+# RCS: @(#) $Id: fileutil.tcl,v 1.39 2004/05/11 19:14:28 afaupell Exp $
 
 package require Tcl 8.2
 package require cmdline
-package provide fileutil 1.5
+package provide fileutil 1.6.1
 
 namespace eval ::fileutil {
     namespace export grep find findByPattern cat foreachLine
@@ -58,7 +58,7 @@ proc ::fileutil::grep {pattern {files {}}} {
 
 # ::fileutil::find ==
 #
-# Two different implementations of this command, one for unix with its
+# Two different implementations of this command, one for Unix with its
 # softlinks, the other for the Win* platform. The trouble with
 # softlink is that they can generate circles in the directory and/or
 # file structure, leading a simple recursion into infinity. So we
@@ -88,10 +88,23 @@ if {[string compare unix $tcl_platform(platform)]} {
     #	files		a list of interesting files.
 
     proc ::fileutil::find {{basedir .} {filtercmd {}}} {
-	set oldwd [pwd]
-	cd $basedir
-	set cwd [pwd]
-	set filenames [glob -nocomplain * .*]
+
+	# Instead of getting a directory, we have received one file
+	# name.  Do not do directory operations.
+	if { [file isfile $basedir] } {
+	    set cwd "" ; # This variable is needed below.
+	    set fileisbasedir 1
+	    set filenames [list $basedir]
+	} elseif { [file isdirectory $basedir] } {
+	    set fileisbasedir 0
+	    set oldwd [pwd]
+	    cd $basedir
+	    set cwd [pwd]
+	    set filenames [glob -nocomplain * .*]
+	} else {
+	    return -code error "$basedir does not exist"
+	}
+
 	set files {}
 	set filt [string length $filtercmd]
 	# If we don't remove . and .. from the file list, we'll get stuck in
@@ -110,77 +123,193 @@ if {[string compare unix $tcl_platform(platform)]} {
 		set files [concat $files [find $filename $filtercmd]]
 	    }
 	}
-	cd $oldwd
+	if { ! $fileisbasedir } {
+	    cd $oldwd
+	}
 	return $files
     }
 } else {
     # Unix, record dev/inode to detect and break circles
 
-    # ::fileutil::find --
-    #
-    #	Implementation of find.  Adapted from the Tcler's Wiki.
-    #
-    # Arguments:
-    #	basedir		directory to start searching from; default is .
-    #	filtercmd	command to use to evaluate interest in each file.
-    #			If NULL, all files are interesting.
-    #
-    # Results:
-    #	files		a list of interesting files.
+    # SF tcllib bug [784157], distinguish between pre and post Tcl
+    # 8.4. In 8.4 and post 8.4. we have to conditionally exclude
+    # dev/inode checking. This is not required for pre 8.4.
 
-    proc ::fileutil::find {{basedir .} {filtercmd {}} {nodeVar {}}} {
-	if {$nodeVar == {}} {
-	    # Main call, setup the device/inode structure
-	    array set inodes {}
-	} else {
-	    # Recursive call, import the device/inode record from the caller.
-	    upvar $nodeVar inodes
+    if {[package vcompare [package present Tcl] 8.4] >= 0} {
+
+	# ::fileutil::find --
+	#
+	#	Implementation of find.  Adapted from the Tcler's Wiki.
+	#
+	# Arguments:
+	#	basedir		directory to start searching from; default is .
+	#	filtercmd	command to use to evaluate interest in each file.
+	#			If NULL, all files are interesting.
+	#
+	# Results:
+	#	files		a list of interesting files.
+
+	proc ::fileutil::find {{basedir .} {filtercmd {}} {nodeVar {}}} {
+	    if {$nodeVar == {}} {
+		# Main call, setup the device/inode structure
+		array set inodes {}
+	    } else {
+		# Recursive call, import the device/inode record from the caller.
+		upvar $nodeVar inodes
+	    }
+
+	    # Instead of getting a directory, we have received one file
+	    # name.  Do not do directory operations.
+	    if { [file isfile $basedir] } {
+		set cwd "" ; # This variable is needed below.
+		set fileisbasedir 1
+		set filenames [list $basedir]
+	    } elseif { [file isdirectory $basedir] } {
+		set fileisbasedir 0
+		set oldwd [pwd]
+		cd $basedir
+		set cwd [pwd]
+		set filenames [glob -nocomplain * .*]
+	    } else {
+		return -code error "$basedir does not exist"
+	    }
+
+	    set files {}
+	    set filt [string length $filtercmd]
+	    # If we don't remove . and .. from the file list, we'll get stuck in
+	    # an infinite loop in an infinite loop in an infinite loop in an inf...
+	    foreach special [list "." ".."] {
+		set index [lsearch -exact $filenames $special]
+		set filenames [lreplace $filenames $index $index]
+	    }
+	    foreach filename $filenames {
+		# Stat each file/directory get exact information about its identity
+		# (device, inode). Non-'stat'able files are either junk (link to
+		# non-existing target) or not readable, i.e. inaccessible. In both
+		# cases it makes sense to ignore them.
+
+		if {[catch {file stat [set full [file join $cwd $filename]] stat}]} {
+		    continue
+		}
+
+		# SF [ 647974 ] find has problems recursing a metakit fs ...
+		#
+		# The following code is a HACK / workaround. We assume that virtual
+		# FS's do not support links, and therefore there is no need for
+		# keeping track of device/inode information. A good thing as the 
+		# the virtual FS's usually give us bad data for these anyway, as
+		# illustrated by the bug referenced above.
+
+		if {[string equal native [lindex [file system $full] 0]]} {
+		    # No skip over previously recorded files/directories and
+		    # record the new files/directories.
+
+		    set key "$stat(dev),$stat(ino)"
+		    if {[info exists inodes($key)]} {
+			continue
+		    }
+		    set inodes($key) 1
+		}
+
+		# Use uplevel to eval the command, not eval, so that variable 
+		# substitutions occur in the right context.
+		if {!$filt || [uplevel $filtercmd [list $filename]]} {
+		    lappend files $full
+		}
+		if {[file isdirectory $filename]} {
+		    set files [concat $files [find $filename $filtercmd inodes]]
+		}
+	    }
+	    if { ! $fileisbasedir } {
+		cd $oldwd
+	    }
+	    return $files
 	}
 
-	set oldwd [pwd]
-	cd $basedir
-	set cwd [pwd]
-	set filenames [glob -nocomplain * .*]
-	set files {}
-	set filt [string length $filtercmd]
-	# If we don't remove . and .. from the file list, we'll get stuck in
-	# an infinite loop in an infinite loop in an infinite loop in an inf...
-	foreach special [list "." ".."] {
-	    set index [lsearch -exact $filenames $special]
-	    set filenames [lreplace $filenames $index $index]
+    } else {
+	# Unix, pre 8.4. No virtual file system is present, therefore there is no
+	# need to conditionally exclude dev/inode checking.
+
+	# ::fileutil::find --
+	#
+	#	Implementation of find.  Adapted from the Tcler's Wiki.
+	#
+	# Arguments:
+	#	basedir		directory to start searching from; default is .
+	#	filtercmd	command to use to evaluate interest in each file.
+	#			If NULL, all files are interesting.
+	#
+	# Results:
+	#	files		a list of interesting files.
+
+	proc ::fileutil::find {{basedir .} {filtercmd {}} {nodeVar {}}} {
+	    if {$nodeVar == {}} {
+		# Main call, setup the device/inode structure
+		array set inodes {}
+	    } else {
+		# Recursive call, import the device/inode record from the caller.
+		upvar $nodeVar inodes
+	    }
+
+	    # Instead of getting a directory, we have received one file
+	    # name.  Do not do directory operations.
+	    if { [file isfile $basedir] } {
+		set cwd "" ; # This variable is needed below.
+		set fileisbasedir 1
+		set filenames [list $basedir]
+	    } elseif { [file isdirectory $basedir] } {
+		set fileisbasedir 0
+		set oldwd [pwd]
+		cd $basedir
+		set cwd [pwd]
+		set filenames [glob -nocomplain * .*]
+	    } else {
+		return -code error "$basedir does not exist"
+	    }
+
+	    set files {}
+	    set filt [string length $filtercmd]
+	    # If we don't remove . and .. from the file list, we'll get stuck in
+	    # an infinite loop in an infinite loop in an infinite loop in an inf...
+	    foreach special [list "." ".."] {
+		set index [lsearch -exact $filenames $special]
+		set filenames [lreplace $filenames $index $index]
+	    }
+	    foreach filename $filenames {
+		# Stat each file/directory get exact information about its identity
+		# (device, inode). Non-'stat'able files are either junk (link to
+		# non-existing target) or not readable, i.e. inaccessible. In both
+		# cases it makes sense to ignore them.
+
+		if {[catch {file stat [set full [file join $cwd $filename]] stat}]} {
+		    continue
+		}
+
+		# No skip over previously recorded files/directories and
+		# record the new files/directories.
+
+		set key "$stat(dev),$stat(ino)"
+		if {[info exists inodes($key)]} {
+		    continue
+		}
+		set inodes($key) 1
+
+		# Use uplevel to eval the command, not eval, so that variable 
+		# substitutions occur in the right context.
+		if {!$filt || [uplevel $filtercmd [list $filename]]} {
+		    lappend files $full
+		}
+		if {[file isdirectory $filename]} {
+		    set files [concat $files [find $filename $filtercmd inodes]]
+		}
+	    }
+	    if { ! $fileisbasedir } {
+		cd $oldwd
+	    }
+	    return $files
 	}
-	foreach filename $filenames {
-	    # Stat each file/directory get exact information about its identity
-	    # (device, inode). Non-'stat'able files are either junk (link to
-	    # non-existing target) or not readable, i.e. inaccessible. In both
-	    # cases it makes sense to ignore them.
 
-	    if {[catch {file stat [file join $cwd $filename] stat}]} {
-		continue
-	    }
-
-	    # No skip over previously recorded files/directories and
-	    # record the new files/directories.
-
-	    set key "$stat(dev),$stat(ino)"
-	    if {[info exists inodes($key)]} {
-		continue
-	    }
-	    set inodes($key) 1
-
-	    # Use uplevel to eval the command, not eval, so that variable 
-	    # substitutions occur in the right context.
-	    if {!$filt || [uplevel $filtercmd [list $filename]]} {
-		lappend files [file join $cwd $filename]
-	    }
-	    if {[file isdirectory $filename]} {
-		set files [concat $files [find $filename $filtercmd inodes]]
-	    }
-	}
-	cd $oldwd
-	return $files
     }
-
     # end if
 }
 
@@ -296,7 +425,7 @@ proc ::fileutil::stripPwd {path} {
     set npath [file split $path]
 
     if {[string match ${pwd}* $npath]} {
-	set path [eval file join [lrange $npath [llength $pwd] end]]
+	set path [eval [linsert [lrange $npath [llength $pwd] end] 0 file join ]]
     }
     return $path
 }
@@ -317,7 +446,7 @@ proc ::fileutil::stripN {path n} {
     if {$n >= [llength $path]} {
 	return {}
     } else {
-	return [eval file join [lrange $path $n end]]
+	return [eval [linsert [lrange $path $n end] 0 file join]]
     }
 }
 
@@ -327,26 +456,28 @@ proc ::fileutil::stripN {path n} {
 #	of the specified file.
 #
 # Arguments:
-#	filename	name of the file to read.
+#	args	name of the files to read.
 #
 # Results:
 #	data		data read from the file.
 
-proc ::fileutil::cat {filename} {
-    # Don't bother catching errors, just let them propagate up
-    set fd [open $filename r]
-    # Use the [file size] command to get the size, which preallocates memory,
-    # rather than trying to grow it as the read progresses.
-    set size [file size $filename]
-    if {$size} {
-        set data [read $fd $size]
-    } else {
-        # if the file has zero bytes it is either empty, or something 
-        # where [file size] reports 0 but the file actually has data (like
-        # the files in the /proc filesystem on Linux)
-        set data [read $fd]
+proc ::fileutil::cat {args} {
+    foreach filename $args {
+	# Don't bother catching errors, just let them propagate up
+	set fd [open $filename r]
+	# Use the [file size] command to get the size, which preallocates memory,
+	# rather than trying to grow it as the read progresses.
+	set size [file size $filename]
+	if {$size} {
+	    append data [read $fd $size]
+	} else {
+	    # if the file has zero bytes it is either empty, or something 
+	    # where [file size] reports 0 but the file actually has data (like
+	    # the files in the /proc filesystem on Linux)
+	    append data [read $fd]
+	}
+	close $fd
     }
-    close $fd
     return $data
 }
 
@@ -539,7 +670,13 @@ proc ::fileutil::fileType {filename} {
         set type text
         set binary 0
     }
-    if { [ regexp {^\#\!(\S+)} $test -> terp ] } {
+
+    # SF Tcllib bug [795585]. Allowing whitespace between #!
+    # and path of script interpreter
+
+    set metakit 0
+
+    if { [ regexp {^\#\!\s*(\S+)} $test -> terp ] } {
         lappend type script $terp
     } elseif { $binary && [ regexp {^[\x7F]ELF} $test ] } {
         lappend type executable elf
@@ -551,8 +688,13 @@ proc ::fileutil::fileType {filename} {
         lappend type graphic gif
     } elseif { $binary && [string match "\x89PNG*" $test] } {
         lappend type graphic png
-    } elseif { $binary && [string match "\xFF\xD8\xFF\xE0\x00\x10JFIF*" $test] } {
-        lappend type graphic jpeg
+    } elseif { $binary && [string match "\xFF\xD8\xFF*" $test] } {
+        binary scan $test c3H2Sa5 id marker len txt 
+        if {$marker == "e0" && $txt == "JFIF\x00"} {
+            lappend type graphic jpeg jfif
+        } elseif { $marker == "e1" && $txt == "Exif\x00" } {
+            lappend type graphic jpeg exif
+        }
     } elseif { $binary && [string match "MM\x00\**" $test] } {
         lappend type graphic tiff
     } elseif { $binary && [string match "\%PDF\-*" $test] } {
@@ -573,7 +715,73 @@ proc ::fileutil::fileType {filename} {
         lappend type message pgp
     } elseif { $binary && [string match {IGWD*} $test] } {
         lappend type gravity_wave_data_frame
-    }    
+    } elseif {[string match "JL\x1a\x00*" $test] && ([file size $filename] >= 27)} {
+	lappend type metakit smallendian
+	set metakit 1
+    } elseif {[string match "LJ\x1a\x00*" $test] && ([file size $filename] >= 27)} {
+	lappend type metakit bigendian
+	set metakit 1
+    }
+
+    # Additional checks of file contents at the end of the file,
+    # possibly pointing into the middle too (attached metakit,
+    # attached zip).
+
+    ## Metakit File format: http://www.equi4.com/metakit/metakit-ff.html
+    ## Metakit database attached ? ##
+
+    if {!$metakit && ([file size $filename] >= 27)} {
+	# The offsets in the footer are in always bigendian format
+
+	if { [ catch {
+	    set fid [ open $filename r ]
+	    fconfigure $fid -translation binary
+	    fconfigure $fid -buffersize 1024
+	    fconfigure $fid -buffering full
+	    seek $fid -16 end
+	    set test [ read $fid 16 ]
+	    ::close $fid
+	} err ] } {
+	    catch { ::close $fid }
+	    return -code error "::fileutil::fileType: $err"
+	}
+
+	binary scan $test IIII __ hdroffset __ __
+	set hdroffset [expr {[file size $filename] - 16 - $hdroffset}]
+
+	# Further checks iff the offset is actually inside the file.
+
+	if {($hdroffset >= 0) && ($hdroffset < [file size $filename])} {
+	    # Seek to the specified location and try to match a metakit header
+	    # at this location.
+
+	    if { [ catch {
+		set         fid [ open $filename r ]
+		fconfigure $fid -translation binary
+		fconfigure $fid -buffersize 1024
+		fconfigure $fid -buffering full
+		seek       $fid $hdroffset start
+		set test [ read $fid 16 ]
+		::close $fid
+	    } err ] } {
+		catch { ::close $fid }
+		return -code error "::fileutil::fileType: $err"
+	    }
+
+	    if {[string match "JL\x1a\x00*" $test]} {
+		lappend type attached metakit smallendian
+		set metakit 1
+	    } elseif {[string match "LJ\x1a\x00*" $test]} {
+		lappend type attached metakit bigendian
+		set metakit 1
+	    }
+	}
+    }
+
+    ## Zip File Format: http://zziplib.sourceforge.net/zzip-parse.html
+    ## http://www.pkware.com/products/enterprise/white_papers/appnote.html
+
+
     ;## lastly, is it a link?
     if { ! [ catch {file readlink $filename} ] } {
         lappend type link
@@ -581,6 +789,79 @@ proc ::fileutil::fileType {filename} {
     return $type
 }
 
+# ::fileutil::tempdir --
+#
+#	Return the correct directory to use for temporary files.
+#	Python attempts this sequence, which seems logical:
+#
+#       1. The directory named by the `TMPDIR' environment variable.
+#
+#       2. The directory named by the `TEMP' environment variable.
+#
+#       3. The directory named by the `TMP' environment variable.
+#
+#       4. A platform-specific location:
+#            * On Macintosh, the `Temporary Items' folder.
+#
+#            * On Windows, the directories `C:\\TEMP', `C:\\TMP',
+#              `\\TEMP', and `\\TMP', in that order.
+#
+#            * On all other platforms, the directories `/tmp',
+#              `/var/tmp', and `/usr/tmp', in that order.
+#
+#        5. As a last resort, the current working directory.
+#
+# Arguments:
+#	None.
+#
+# Side Effects:
+#	None.
+#
+# Results:
+#	The directory for temporary files.
+
+proc ::fileutil::TempDir {} {
+    global tcl_platform env
+    set attempdirs [list]
+
+    foreach tmp {TMPDIR TEMP TMP} {
+	if { [info exists env($tmp)] } {
+	    lappend attempdirs $env($tmp)
+	}
+    }
+
+    switch $tcl_platform(platform) {
+	windows {
+	    lappend attempdirs "C:\\TEMP" "C:\\TMP" "\\TEMP" "\\TMP"
+	}
+	macintosh {
+	    set tmpdir $env(TRASH_FOLDER)  ;# a better place?
+	}
+	default {
+	    lappend attempdirs [file join / tmp] \
+		[file join / var tmp] [file join / usr tmp]
+	}
+    }
+
+    foreach tmp $attempdirs {
+	if { [file isdirectory $tmp] && [file writable $tmp] } {
+	    return $tmp
+	}
+    }
+
+    # If nothing else worked...
+    return [pwd]
+}
+
+if { [package vcompare [package provide Tcl] 8.4] < 0 } {
+    proc ::fileutil::tempdir {} {
+	return [TempDir]
+    }
+} else {
+    proc ::fileutil::tempdir {} {
+	return [file normalize [TempDir]]
+    }
+}
 
 # ::fileutil::tempfile --
 #
@@ -596,19 +877,8 @@ proc ::fileutil::fileType {filename} {
 #   returns a file name
 #
 
-proc ::fileutil::tempfile {{prefix {}}} {
-    global  tcl_platform
-    switch $tcl_platform(platform) {
-	unix {
-	    set tmpdir /tmp;   # or even $::env(TMPDIR), at times.
-	} macintosh {
-	    set tmpdir $env(TRASH_FOLDER)  ;# a better place?
-	} default {
-	    set tmpdir [pwd]
-	    catch {set tmpdir $env(TMP)}
-	    catch {set tmpdir $env(TEMP)}
-	}
-    }
+proc ::fileutil::TempFile {prefix} {
+    set tmpdir [tempdir]
 
     set chars "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
     set nrand_chars 10
@@ -622,7 +892,7 @@ proc ::fileutil::tempfile {{prefix {}}} {
  	set newname $prefix
  	for {set j 0} {$j < $nrand_chars} {incr j} {
  	    append newname [string index $chars \
-				[expr ([clock clicks] ^ $mypid) % 62]]
+		    [expr {([clock clicks] ^ $mypid) % 62}]]
  	}
 	set newname [file join $tmpdir $newname]
  	if {[file exists $newname]} {
@@ -632,7 +902,7 @@ proc ::fileutil::tempfile {{prefix {}}} {
  		if {!$checked_dir_writable} {
  		    set dirname [file dirname $newname]
  		    if {![file writable $dirname]} {
- 			error "Directory $dirname is not writable"
+ 			return -code error "Directory $dirname is not writable"
  		    }
  		    set checked_dir_writable 1
  		}
@@ -649,3 +919,51 @@ proc ::fileutil::tempfile {{prefix {}}} {
  	return -code error "Failed to find an unused temporary file name"
     }
 }
+
+if { [package vcompare [package provide Tcl] 8.4] < 0 } {
+    proc ::fileutil::tempfile {{prefix {}}} {
+	return [TempFile $prefix]
+    }
+} else {
+    proc ::fileutil::tempfile {{prefix {}}} {
+	return [file normalize [TempFile $prefix]]
+    }
+}
+
+# ::fileutil::install --
+#
+#	Tcl version of the 'install' command, which copies files from
+#	one places to another and also optionally sets some attributes
+#	such as group, owner, and permissions.
+#
+# Arguments:
+#	-m		Change the file permissions to the specified
+#                       value.  Valid arguments are those accepted by
+#			file attributes -permissions
+#
+# Results:
+#	None.
+
+# TODO - add options for group/owner manipulation.
+
+proc ::fileutil::install {args} {
+    set options {
+	{m.arg "" "Set permission mode"}
+    }
+    set usage ": [lindex [info level 0] 0]\
+\[options] source destination \noptions:"
+    array set params [::cmdline::getoptions args $options $usage]
+    # Args should now just be the source and destination.
+    if { [llength $args] < 2 } {
+	return -code error $usage
+    }
+    set src [lindex $args 0]
+    set dst [lindex $args 1]
+    file copy -force $src $dst
+    if { $params(m) != "" } {
+	set targets [::fileutil::find $dst]
+	foreach fl $targets {
+	    file attributes $fl -permissions $params(m)
+	}
+    }
+}
