@@ -13,7 +13,7 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id: ftp.tcl,v 1.23 2002/01/18 20:51:16 andreas_kupries Exp $
+# RCS: @(#) $Id$
 #
 #   core ftp support: 	ftp::Open <server> <user> <passwd> <?options?>
 #			ftp::Close <s>
@@ -23,7 +23,7 @@
 #			ftp::List <s> <?directory?>
 #			ftp::NList <s> <?directory?>
 #			ftp::FileSize <s> <file>
-#			ftp::ModTime <s> <from> <to>
+#			ftp::ModTime <s> <file> <?newtime?>
 #			ftp::Delete <s> <file>
 #			ftp::Rename <s> <from> <to>
 #			ftp::Put <s> <(local | -data "data" -channel chan)> <?remote?>
@@ -53,11 +53,11 @@
 package require Tcl 8.2
 package require log     ; # tcllib/log, general logging facility.
 
-namespace eval ftp {
+namespace eval ::ftp {
     namespace export DisplayMsg Open Close Cd Pwd Type List NList \
 	    FileSize ModTime Delete Rename Put Append Get Reget \
 	    Newer Quote MkDir RmDir
-	
+
     set serial 0
     set VERBOSE 0
     set DEBUG 0
@@ -80,10 +80,10 @@ namespace eval ftp {
 # msg - 		message string
 # state -		different states {normal, data, control, error}
 #
-proc ftp::DisplayMsg {s msg {state ""}} {
+proc ::ftp::DisplayMsg {s msg {state ""}} {
 
     upvar ::ftp::ftp$s ftp
-    
+
     if { ([info exists ftp(Output)]) && ($ftp(Output) != "") } {
         eval [concat $ftp(Output) {$s $msg $state}]
         return
@@ -98,7 +98,7 @@ proc ftp::DisplayMsg {s msg {state ""}} {
     #              needs of any application using the ftp module. The
     #              variable VERBOSE is still relevant as it controls
     #              whether this procedure is called or not.
-        
+
     switch -exact -- $state {
         data    {log::log debug "$state | $msg"}
         control {log::log debug "$state | $msg"}
@@ -117,7 +117,7 @@ proc ftp::DisplayMsg {s msg {state ""}} {
 # Arguments:
 #  -
 #
-proc ftp::Timeout {s} {
+proc ::ftp::Timeout {s} {
     upvar ::ftp::ftp$s ftp
 
     after cancel $ftp(Wait)
@@ -143,7 +143,7 @@ proc ftp::Timeout {s} {
 #  -		
 #
 
-proc ftp::WaitOrTimeout {s} {
+proc ::ftp::WaitOrTimeout {s} {
     upvar ::ftp::ftp$s ftp
 
     set retvar 1
@@ -178,7 +178,7 @@ proc ftp::WaitOrTimeout {s} {
 #			1 ... OK
 #
 
-proc ftp::WaitComplete {s value} {
+proc ::ftp::WaitComplete {s value} {
     upvar ::ftp::ftp$s ftp
 
     if {![info exists ftp(Command)]} {
@@ -205,7 +205,7 @@ proc ftp::WaitComplete {s value} {
 # command - 		ftp command
 #
 
-proc ftp::PutsCtrlSock {s {command ""}} {
+proc ::ftp::PutsCtrlSock {s {command ""}} {
     upvar ::ftp::ftp$s ftp
     variable DEBUG
 	
@@ -231,7 +231,7 @@ proc ftp::PutsCtrlSock {s {command ""}} {
 # 			If called from a fileevent than this argument contains
 #			the socket channel identifier.
 
-proc ftp::StateHandler {s {sock ""}} {
+proc ::ftp::StateHandler {s {sock ""}} {
     upvar ::ftp::ftp$s ftp
     variable DEBUG 
     variable VERBOSE
@@ -252,19 +252,26 @@ proc ftp::StateHandler {s {sock ""}} {
 
             # get return code, check for multi-line text
             
-            regexp -- "(^\[0-9\]+)( |-)?(.*)$" $bufline all rc multi_line msgtext
-            set buffer $bufline
+            if {![regexp -- "^-?(\[0-9\]+)( |-)?(.*)$" $bufline all rc multi_line msgtext]} {
+		set errmsg "C: Internal Error @ line 255.\
+			Regex pattern not matching the input \"$bufline\""
+		if {$VERBOSE} {
+		    DisplayMsg $s $errmsg control
+		}
+	    } else {
+		set buffer $bufline
 			
-            # multi-line format detected ("-"), get all the lines
-            # until the real return code
+		# multi-line format detected ("-"), get all the lines
+		# until the real return code
 
-            while { [string equal $multi_line "-"] } {
-                set number [gets $sock bufline]	
-                if { $number > 0 } {
-                    append buffer \n "$bufline"
-                    regexp -- "(^\[0-9\]+)( |-)?(.*)$" $bufline all rc multi_line
-                }
-            }
+		while { [string equal $multi_line "-"] } {
+		    set number [gets $sock bufline]	
+		    if { $number > 0 } {
+			append buffer \n "$bufline"
+			regexp -- "(^\[0-9\]+)( |-)?(.*)$" $bufline all rc multi_line
+		    }
+		}
+	    }
         } elseif { [eof $ftp(CtrlSock)] } {
             # remote server has closed control connection
             # kill control socket, unset State to disable all following command
@@ -547,7 +554,11 @@ proc ftp::StateHandler {s {sock ""}} {
             }
         } 
         modtime {
-            PutsCtrlSock $s "MDTM $ftp(File)"
+            if {$ftp(DateTime) != ""} {
+              PutsCtrlSock $s "MDTM $ftp(DateTime) $ftp(File)"
+            } else { ;# No DateTime Specified
+              PutsCtrlSock $s "MDTM $ftp(File)"
+            }
             set ftp(State) modtime_sent
         }  
         modtime_sent {
@@ -559,7 +570,11 @@ proc ftp::StateHandler {s {sock ""}} {
 		    Command $ftp(Command) modtime $ftp(File) [ModTimePostProcess $ftp(DateTime)]
                 }
                 default {
-                    set errmsg "Error getting modification time!"
+                    if {$ftp(DateTime) != ""} {
+                      set errmsg "Error setting modification time! No server MDTM support?"
+                    } else {
+                      set errmsg "Error getting modification time!"
+                    }
                     set complete_with 0
 		    Command $ftp(Command) error $errmsg
                 }
@@ -1045,7 +1060,7 @@ proc ftp::StateHandler {s {sock ""}} {
 # Returns:
 # type	-		returns the current type or {} if an error occurs
 
-proc ftp::Type {s {type ""}} {
+proc ::ftp::Type {s {type ""}} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1099,7 +1114,7 @@ proc ftp::Type {s {type ""}} {
 # Returns:
 # sorted list of files or {} if listing fails
 
-proc ftp::NList {s { dir ""}} {
+proc ::ftp::NList {s { dir ""}} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1169,7 +1184,7 @@ proc ftp::NList {s { dir ""}} {
 # Returns:
 # list of files or {} if listing fails
 
-proc ftp::List {s {dir ""}} {
+proc ::ftp::List {s {dir ""}} {
 
     upvar ::ftp::ftp$s ftp
 
@@ -1224,7 +1239,7 @@ proc ftp::List {s {dir ""}} {
     }
 }
 
-proc ftp::ListPostProcess l {
+proc ::ftp::ListPostProcess l {
 
     # clear "total"-line
 
@@ -1259,7 +1274,7 @@ proc ftp::ListPostProcess l {
 # Returns:
 # size -		files size in bytes or {} in error cases
 
-proc ftp::FileSize {s {filename ""}} {
+proc ::ftp::FileSize {s {filename ""}} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1306,13 +1321,15 @@ proc ftp::FileSize {s {filename ""}} {
 # 
 # Arguments:
 # filename - 		specifies the remote file name
+# datetime -            optional new timestamp for file
 # 
 # Returns:
 # clock -		files date and time as a system-depentend integer
 #			value in seconds (see tcls clock command) or {} in 
 #			error cases
+# if MDTM not supported on server, returns original timestamp
 
-proc ftp::ModTime {s {filename ""}} {
+proc ::ftp::ModTime {s {filename ""} {datetime ""}} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1329,7 +1346,11 @@ proc ftp::ModTime {s {filename ""}} {
     } 
 
     set ftp(File) $filename
-    set ftp(DateTime) ""
+
+    if {$datetime != ""} {
+      set datetime [clock format $datetime -format "%Y%m%d%H%M%S"]
+    }
+    set ftp(DateTime) $datetime
 	
     set ftp(State) modtime
     StateHandler $s
@@ -1340,7 +1361,6 @@ proc ftp::ModTime {s {filename ""}} {
     if {![string length $ftp(Command)]} {
 	unset ftp(File)
     }
-		
     if { ![string length $ftp(Command)] && $rc } {
         return [ModTimePostProcess $ftp(DateTime)]
     } else {
@@ -1348,7 +1368,7 @@ proc ftp::ModTime {s {filename ""}} {
     }
 }
 
-proc ftp::ModTimePostProcess {clock} {
+proc ::ftp::ModTimePostProcess {clock} {
     foreach {year month day hour min sec} {1 1 1 1 1 1} break
 
     # Bug #478478. Special code to detect ftp servers with a Y2K patch
@@ -1379,7 +1399,7 @@ proc ftp::ModTimePostProcess {clock} {
 # Returns:
 # current directory name
 
-proc ftp::Pwd {s } {
+proc ::ftp::Pwd {s } {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1420,7 +1440,7 @@ proc ftp::Pwd {s } {
 # 0 -			ERROR
 # 1 - 			OK
 
-proc ftp::Cd {s {dir ""}} {
+proc ::ftp::Cd {s {dir ""}} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1471,7 +1491,7 @@ proc ftp::Cd {s {dir ""}} {
 # 0 -			ERROR
 # 1 - 			OK
 
-proc ftp::MkDir {s dir} {
+proc ::ftp::MkDir {s dir} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1514,7 +1534,7 @@ proc ftp::MkDir {s dir} {
 # 0 -			ERROR
 # 1 - 			OK
 
-proc ftp::RmDir {s dir} {
+proc ::ftp::RmDir {s dir} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1556,7 +1576,7 @@ proc ftp::RmDir {s dir} {
 # 0 -			ERROR
 # 1 - 			OK
 
-proc ftp::Delete {s file} {
+proc ::ftp::Delete {s file} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1600,7 +1620,7 @@ proc ftp::Delete {s file} {
 # 0 -			ERROR
 # 1 - 			OK
 
-proc ftp::Rename {s from to} {
+proc ::ftp::Rename {s from to} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1639,7 +1659,8 @@ proc ftp::Rename {s from to} {
 # Arguments:
 # stop_time - 		ending time
 
-proc ftp::ElapsedTime {s stop_time} {
+proc ::ftp::ElapsedTime {s stop_time} {
+    variable VERBOSE
     upvar ::ftp::ftp$s ftp
 
     set elapsed [expr {$stop_time - $ftp(Start_Time)}]
@@ -1647,7 +1668,9 @@ proc ftp::ElapsedTime {s stop_time} {
         set elapsed 1
     }
     set persec [expr {$ftp(Total) / $elapsed}]
-    DisplayMsg $s "$ftp(Total) bytes sent in $elapsed seconds ($persec Bytes/s)"
+    if { $VERBOSE } {
+        DisplayMsg $s "$ftp(Total) bytes sent in $elapsed seconds ($persec Bytes/s)"
+    }
     return
 }
 
@@ -1670,7 +1693,7 @@ proc ftp::ElapsedTime {s stop_time} {
 # 0 -			file not stored
 # 1 - 			OK
 
-proc ftp::Put {s args} {
+proc ::ftp::Put {s args} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1785,7 +1808,7 @@ proc ftp::Put {s args} {
 # 0 -			file not stored
 # 1 - 			OK
 
-proc ftp::Append {s args} {
+proc ::ftp::Append {s args} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1900,7 +1923,7 @@ proc ftp::Append {s args} {
 # 0 -			file not retrieved
 # 1 - 			OK
 
-proc ftp::Get {s args} {
+proc ::ftp::Get {s args} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -1943,7 +1966,7 @@ proc ftp::Get {s args} {
     }
 
     if {($ftp(inline) != 0) && ($dest != "")} {
-        DisplayMsg $s "Cannot return data in a variable, and place it in destination file." error
+        DisplayMsg $s "Cannot return data in a variable or channel, and place it in destination file." error
         return 0
     }
 
@@ -1952,7 +1975,7 @@ proc ftp::Get {s args} {
         return 0
     }
 
-    if {$ftp(inline) != 2} {
+    if {$ftp(inline) == 0} {
 	if { $dest == "" } {
 	    set dest $source
 	} else {
@@ -2023,7 +2046,7 @@ proc ftp::Get {s args} {
 # 0 -			file not retrieved
 # 1 - 			OK
 
-proc ftp::Reget {s source {dest ""} {from_bytes 0} {till_bytes -1}} {
+proc ::ftp::Reget {s source {dest ""} {from_bytes 0} {till_bytes -1}} {
     upvar ::ftp::ftp$s ftp
     
     if { ![info exists ftp(State)] } {
@@ -2035,7 +2058,8 @@ proc ftp::Reget {s source {dest ""} {from_bytes 0} {till_bytes -1}} {
         set dest $source
     }
     if {![file exists [file dirname $dest]]} {
-	return -code error "ftp::Reget, directory \"[file dirname $dest]\" for destination \"$dest\" does not exist"
+	return -code error \
+	"ftp::Reget, directory \"[file dirname $dest]\" for destination \"$dest\" does not exist"
     }
 
     set ftp(RemoteFilename) $source
@@ -2048,13 +2072,13 @@ proc ftp::Reget {s source {dest ""} {from_bytes 0} {till_bytes -1}} {
     # correct offset
 
     if { [file exists $ftp(LocalFilename)] } {
-	set ftp(FileSize) [ expr [file size $ftp(LocalFilename)] + $from_bytes ]
+	set ftp(FileSize) [ expr {[file size $ftp(LocalFilename)] + $from_bytes }]
 	 	
 	if { $till_bytes != -1 } {
 	    set ftp(To)   $till_bytes	
-	    set ftp(Bytes_to_go) [ expr $till_bytes - $ftp(FileSize) ]
+	    set ftp(Bytes_to_go) [ expr {$till_bytes - $ftp(FileSize)} ]
 	
-	    if { $ftp(Bytes_to_go) <= 0 } return 0
+	    if { $ftp(Bytes_to_go) <= 0 } {return 0}
 
 	} else {
 	    # till_bytes not set
@@ -2066,7 +2090,7 @@ proc ftp::Reget {s source {dest ""} {from_bytes 0} {till_bytes -1}} {
         set ftp(FileSize) $from_bytes
 		  
 	if { $till_bytes != -1 } {
-	    set ftp(Bytes_to_go) [ expr $till_bytes - $from_bytes ]
+	    set ftp(Bytes_to_go) [ expr {$till_bytes - $from_bytes }]
 	    set ftp(To) $till_bytes
 	} else {
 	    #till_bytes not set
@@ -2109,7 +2133,7 @@ proc ftp::Reget {s source {dest ""} {from_bytes 0} {till_bytes -1}} {
 # 0 -			file not retrieved
 # 1 - 			OK
 
-proc ftp::Newer {s source {dest ""}} {
+proc ::ftp::Newer {s source {dest ""}} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -2169,7 +2193,7 @@ proc ftp::Newer {s source {dest ""}} {
 #  string sent back by the remote ftp server or null string if any error
 #
 
-proc ftp::Quote {s args} {
+proc ::ftp::Quote {s args} {
     upvar ::ftp::ftp$s ftp
 
     if { ![info exists ftp(State)] } {
@@ -2235,7 +2259,7 @@ proc ftp::Quote {s args} {
 # 0 -			ERROR
 # 1 - 			OK
 
-proc ftp::Close {s } {
+proc ::ftp::Close {s } {
     variable connections
     upvar ::ftp::ftp$s ftp
 
@@ -2261,7 +2285,7 @@ proc ftp::Close {s } {
     return 1
 }
 
-proc ftp::LazyClose {s } {
+proc ::ftp::LazyClose {s } {
     variable connections
     upvar ::ftp::ftp$s ftp
 
@@ -2309,7 +2333,7 @@ proc ftp::LazyClose {s } {
 # 0 -			Not logged in
 # 1 - 			User logged in
 
-proc ftp::Open {server user passwd args} {
+proc ::ftp::Open {server user passwd args} {
     variable DEBUG 
     variable VERBOSE
     variable serial
@@ -2421,7 +2445,7 @@ proc ftp::Open {server user passwd args} {
 # Arguments:
 # bytes - 		indicates how many bytes were written on $ftp(DestCI)
 
-proc ftp::CopyNext {s bytes {error {}}} {
+proc ::ftp::CopyNext {s bytes {error {}}} {
     upvar ::ftp::ftp$s ftp
     variable DEBUG
     variable VERBOSE
@@ -2433,7 +2457,7 @@ proc ftp::CopyNext {s bytes {error {}}} {
     # update bytes_to_go and blocksize
 
     if { [info exists ftp(Bytes_to_go)] } {
-	set ftp(Bytes_to_go) [expr $ftp(Bytes_to_go) - $bytes]
+	set ftp(Bytes_to_go) [expr {$ftp(Bytes_to_go) - $bytes}]
 	 
 	if { $ftp(Blocksize) <= $ftp(Bytes_to_go) } {
 	    set blocksize $ftp(Blocksize)
@@ -2502,7 +2526,7 @@ proc ftp::CopyNext {s bytes {error {}}} {
 # Arguments:
 # sock - 		socket name (data channel)
 
-proc ftp::HandleData {s sock} {
+proc ::ftp::HandleData {s sock} {
     upvar ::ftp::ftp$s ftp
 
     # Turn off any fileevent handlers
@@ -2512,7 +2536,7 @@ proc ftp::HandleData {s sock} {
 
     # create local file for ftp::Get 
 
-    if { [regexp -- "^get" $ftp(State)]  && (!$ftp(inline))} {
+    if { [string match "get*" $ftp(State)]  && (!$ftp(inline))} {
 
 	# A channel was specified by the caller. Use that instead of a
 	# file.
@@ -2537,7 +2561,7 @@ proc ftp::HandleData {s sock} {
 
     # append local file for ftp::Reget 
 
-    if { [regexp -- "^reget" $ftp(State)] } {
+    if { [string match "reget*" $ftp(State)] } {
         set rc [catch {set ftp(DestCI) [open $ftp(LocalFilename) a]} msg]
         if { $rc != 0 } {
             DisplayMsg $s "$msg" error
@@ -2585,7 +2609,7 @@ proc ftp::HandleData {s sock} {
 # Arguments:
 # sock - 		socket name (data channel)
 
-proc ftp::HandleList {s sock} {
+proc ::ftp::HandleList {s sock} {
     upvar ::ftp::ftp$s ftp
     variable VERBOSE
 
@@ -2614,7 +2638,7 @@ proc ftp::HandleList {s sock} {
 # Arguments:
 # sock - 		socket name (data channel)
 
-proc ftp::HandleVar {s sock} {
+proc ::ftp::HandleVar {s sock} {
     upvar ::ftp::ftp$s ftp
     variable VERBOSE
 
@@ -2648,7 +2672,7 @@ proc ftp::HandleVar {s sock} {
 # Arguments:
 # sock - 		socket name (data channel)
 
-proc ftp::HandleOutput {s sock} {
+proc ::ftp::HandleOutput {s sock} {
     upvar ::ftp::ftp$s ftp
     variable VERBOSE
 
@@ -2691,7 +2715,7 @@ proc ftp::HandleOutput {s sock} {
 # Returns:
 # None.
 #
-proc ftp::CloseDataConn {s } {
+proc ::ftp::CloseDataConn {s } {
     upvar ::ftp::ftp$s ftp
 
     # Protect the destination channel from destruction if it came
@@ -2725,7 +2749,7 @@ proc ftp::CloseDataConn {s } {
 #			of the client's host,
 # port -		the client's port number
 
-proc ftp::InitDataConn {s sock addr port} {
+proc ::ftp::InitDataConn {s sock addr port} {
     upvar ::ftp::ftp$s ftp
     variable VERBOSE
 
@@ -2750,10 +2774,11 @@ proc ftp::InitDataConn {s sock addr port} {
 
     # assign fileevent handlers, source and destination CI (Channel Identifier)
 
+    # NB: this really does need to be -regexp [PT] 18Mar03
     switch -regexp -- $ftp(State) {
         list {
             fileevent $sock readable [list [namespace current]::HandleList $s $sock]
-            set ftp(SourceCI) $sock		  
+            set ftp(SourceCI) $sock
         }
         get {
             if {$ftp(inline)} {
@@ -2764,7 +2789,7 @@ proc ftp::InitDataConn {s sock addr port} {
 	    } else {
                 fileevent $sock readable [list [namespace current]::HandleData $s $sock]
                 set ftp(SourceCI) $sock
-	    }			  
+	    }
         }
         append -
         put {
@@ -2775,7 +2800,7 @@ proc ftp::InitDataConn {s sock addr port} {
 	    } else {
                 fileevent $sock writable [list [namespace current]::HandleData $s $sock]
                 set ftp(DestCI) $sock
-	    }			  
+	    }
         }
 	default {
 	    error "Unknown state \"$ftp(State)\""
@@ -2801,7 +2826,7 @@ proc ftp::InitDataConn {s sock addr port} {
 # 0 -			no connection
 # 1 - 			connection established
 
-proc ftp::OpenActiveConn {s } {
+proc ::ftp::OpenActiveConn {s } {
     upvar ::ftp::ftp$s ftp
     variable VERBOSE
 
@@ -2839,7 +2864,7 @@ proc ftp::OpenActiveConn {s } {
 # 0 -			no connection
 # 1 - 			connection established
 
-proc ftp::OpenPassiveConn {s buffer} {
+proc ::ftp::OpenPassiveConn {s buffer} {
     upvar ::ftp::ftp$s ftp
 
     if { [regexp -- {([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+)} $buffer all a1 a2 a3 a4 p1 p2] } {
@@ -2875,7 +2900,7 @@ proc ftp::OpenPassiveConn {s buffer} {
 # 0 -			no connection
 # 1 - 			connection established
 
-proc ftp::OpenControlConn {s {block 1}} {
+proc ::ftp::OpenControlConn {s {block 1}} {
     upvar ::ftp::ftp$s ftp
     variable DEBUG
     variable VERBOSE
@@ -2903,7 +2928,7 @@ proc ftp::OpenControlConn {s {block 1}} {
     # to comma format)
 
     set ftp(LocalAddr) [lindex [fconfigure $ftp(CtrlSock) -sockname] 0]
-    regsub -all -- "\[.\]" $ftp(LocalAddr) "," ftp(LocalAddr) 
+    set ftp(LocalAddr) [string map {. ,} $ftp(LocalAddr)]
 
     # report ready message
 
@@ -2915,7 +2940,7 @@ proc ftp::OpenControlConn {s {block 1}} {
     return 1
 }
 
-# ftp::Command --
+# ::ftp::Command --
 #
 #	Wrapper for evaluated user-supplied command callback
 #
@@ -2927,7 +2952,7 @@ proc ftp::OpenControlConn {s {block 1}} {
 # Results:
 #	Depends on callback script
 
-proc ftp::Command {cb msg args} {
+proc ::ftp::Command {cb msg args} {
     if {[string length $cb]} {
 	uplevel #0 $cb [list $msg] $args
     }
@@ -2951,7 +2976,7 @@ if { [string equal [uplevel "#0" {info commands tkcon}] "tkcon"] } {
     rename ::ftp::List ::ftp::List_org
 
     alias ::ftp::List	::ftp::__ftp_ls
-    alias bye		catch {::ftp::Close; exit}	
+    alias bye		catch {::ftp::Close; exit}
 
     set ::ftp::VERBOSE 1
     set ::ftp::DEBUG 0
@@ -2960,4 +2985,4 @@ if { [string equal [uplevel "#0" {info commands tkcon}] "tkcon"] } {
 # ==================================================================
 # At last, everything is fine, we can provide the package.
 
-package provide ftp [lindex {Revision: 2.3} 1]
+package provide ftp [lindex {Revision: 2.4.1} 1]
