@@ -9,10 +9,10 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id: ftpd.tcl,v 1.12 2001/09/24 19:05:14 jenglish Exp $
+# RCS: @(#) $Id: ftpd.tcl,v 1.24 2004/05/23 18:17:17 gwlester Exp $
 #
 
-# Define the ftpd package version 1.1.1
+# Define the ftpd package version 1.1.2
 
 package require Tcl 8.2
 namespace eval ::ftpd {
@@ -22,35 +22,37 @@ namespace eval ::ftpd {
     variable port 21
 
     variable contact
-    if {![info exist contact]} {
+    if {![info exists contact]} {
         global tcl_platform
 	set contact "$tcl_platform(user)@[info hostname]"
     }
 
     variable cwd
-    if {![info exist cwd]} {
+    if {![info exists cwd]} {
 	set cwd ""
     }
     
     variable welcome
-    if {![info exist welcome]} {
+    if {![info exists welcome]} {
 	set welcome "[info hostname] FTP server ready."
     }
 
     # Global configuration.
 
     variable cfg
-    if {![info exist cfg]} {
+    if {![info exists cfg]} {
 	array set cfg [list \
+	    closeCmd  {} \
 	    authIpCmd  {} \
 	    authUsrCmd {::ftpd::anonAuth} \
             authFileCmd {::ftpd::fileAuth} \
 	    logCmd     {::ftpd::logStderr} \
-	    fsCmd      {::ftpd::fsFile::fs}]
+	    fsCmd      {::ftpd::fsFile::fs} \
+	    xferDoneCmd  {}]
     }
 
     variable commands
-    if {![info exist commands]} {
+    if {![info exists commands]} {
 	array set commands [list \
 	    ABOR       {ABOR (abort operation)} \
 	    ACCT       {(specify account); unimplemented.} \
@@ -107,6 +109,7 @@ namespace eval ::ftpd {
     namespace export fileAuth anonAuth unixAuth server accept read
 }
 
+
 # ::ftpd::config --
 #
 #       Configure the configurable parameters of the ftp daemon.
@@ -144,14 +147,17 @@ proc ::ftpd::config {args} {
     variable cfg
 
     array set cfg [cmdline::getoptions args [list \
+	{closeCmd.arg {} {Callback when a connection is closed.}} \
 	{authIpCmd.arg  {} {Callback to authenticate new connections based on the ip-address of the peer. Optional}} \
 	{authUsrCmd.arg {::ftpd::anonAuth} {Callback to authenticate new connections based on the user logging in.}} \
 	{authFileCmd.arg {::ftpd::fileAuth} {Callback to accept or deny a users access to read and write to a specific path or file.}} \
 	{logCmd.arg {::ftpd::logStderr} {Callback for log information generated by the FTP engine.}} \
+	{xferDoneCmd.arg {} {Callback for transfer completion notification. Optional}} \
 	{fsCmd.arg {::ftpd::fsFile::fs} {Callback to connect the engine to the filesystem it operates on.}}]]
     return
 }
 
+
 # ::ftpd::hasCallback --
 #
 #       Determines whether or not a non-NULL callback has been defined for one
@@ -173,6 +179,7 @@ proc ::ftpd::hasCallback {callbackType} {
     return [expr {[info exists cfg($callbackType)] && [string length $cfg($callbackType)]}]
 }
 
+
 # ::ftpd::logStderr --
 #
 #       Outputs a message with the specified severity to stderr.  The default
@@ -197,6 +204,7 @@ proc ::ftpd::logStderr {severity text} {
     return
 }
 
+
 # ::ftpd::Log --
 #
 #       Used for all ftpd logging.
@@ -213,7 +221,7 @@ proc ::ftpd::logStderr {severity text} {
 #       The ftpd logCmd callback is called with the specified severity and
 #       text if there is a non-NULL ftpCmd.
 
-proc ftpd::Log {severity text} {
+proc ::ftpd::Log {severity text} {
 
     # Central call out to log handlers.
 
@@ -227,6 +235,7 @@ proc ftpd::Log {severity text} {
     return
 }
 
+
 # ::ftpd::fileAuth --
 #
 #       Given a username, path, and operation- decides whether or not to accept
@@ -410,10 +419,11 @@ proc ::ftpd::unixAuth {user pass} {
 proc ::ftpd::server {{myaddr {}}} {
     variable port
     if {[string length $myaddr]} {
-	socket -server ::ftpd::accept -myaddr $myaddr $port
+	set f [socket -server ::ftpd::accept -myaddr $myaddr $port]
     } else {
-	socket -server ::ftpd::accept $port
+	set f [socket -server ::ftpd::accept $port]
     }
+    set port [lindex [fconfigure $f -sockname] 2]
     return
 }
 
@@ -441,8 +451,10 @@ proc ::ftpd::accept {sock ipaddr client_port} {
     variable welcome
     variable cfg
     variable cwd
+    variable CurrentSocket 
 
-    if {[info exist data]} {
+    set CurrentSocket $sock
+    if {[info exists data]} {
 	unset data
     }
 
@@ -476,10 +488,12 @@ proc ::ftpd::accept {sock ipaddr client_port} {
 
     array set data [list \
         access          0 \
+	ip              $ipaddr \
 	state		command \
 	buffering	line \
 	cwd		"$cwd" \
 	mode		binary \
+	sock2a          "" \
         sock2           ""]
 
     fconfigure $sock -buffering line
@@ -510,7 +524,9 @@ proc ::ftpd::accept {sock ipaddr client_port} {
 
 proc ::ftpd::read {sock} {
     upvar #0 ::ftpd::$sock data
+    variable CurrentSocket 
 
+    set CurrentSocket $sock
     if {[eof $sock]} {
 	Finish $sock
 	return
@@ -562,7 +578,19 @@ proc ::ftpd::read {sock} {
 #       The channel is closed.
 
 proc ::ftpd::Finish {sock} {
+    upvar #0 ::ftpd::$sock data
+    variable cfg
+
+    if {[hasCallback closeCmd]} then {
+	##
+	## User specified a close command so invoke it
+	##
+	uplevel #0 $cfg(closeCmd)
+    }
     close $sock
+    if {[info exists data]} {
+	unset data
+    }
     return
 }
 
@@ -713,9 +741,9 @@ proc ::ftpd::command::APPE {sock list} {
         }
     }
 
-#
-# Patched Mark O'Connor
-#
+    #
+    # Patched Mark O'Connor
+    #
     if {![catch {::ftpd::Fs append $path $data(mode)} f]} {
 	puts $sock "150 Copy Started ($data(mode))"
 	fcopy $data(sock2) $f -command [list ::ftpd::GetDone $sock $data(sock2) $f ""]
@@ -1046,7 +1074,7 @@ proc ::ftpd::command::PASS {sock list} {
         unset data(pass)
         puts $sock "551 Access Denied"
     } else {
-	puts $sock "200 OK"
+	puts $sock "230 OK"
 	set data(access) 1
     }
     return
@@ -1119,9 +1147,9 @@ proc ::ftpd::command::PWD {sock list} {
 proc ::ftpd::command::QUIT {sock list} {
     ::ftpd::Log note "Closed $sock"
     puts $sock "221 Goodbye."
-    close $sock
+    ::ftpd::Finish $sock
     # FRINK: nocheck
-    unset ::ftpd::$sock
+    #unset ::ftpd::$sock
     return
 }
 
@@ -1147,6 +1175,7 @@ proc ::ftpd::command::REIN {sock list} {
     upvar #0 ::ftpd::$sock data
 
     ::ftpd::FinishData $sock
+    catch {close $data(sock2a)}
 
     # Reinitialize the user and connection data.
 
@@ -1156,6 +1185,7 @@ proc ::ftpd::command::REIN {sock list} {
 	buffering	line \
 	cwd		"$::ftpd::cwd" \
 	mode		binary \
+	sock2a          "" \
         sock2           ""]
 
     return
@@ -1193,9 +1223,9 @@ proc ::ftpd::command::RETR {sock list} {
         }
     }
 
-#
-# Patched Mark O'Connor
-#
+    #
+    # Patched Mark O'Connor
+    #
     if {![catch {::ftpd::Fs retr $path $data(mode)} f]} {
 	puts $sock "150 Copy Started ($data(mode))"
 	fcopy $f $data(sock2) -command [list ::ftpd::GetDone $sock $data(sock2) $f ""]
@@ -1385,9 +1415,9 @@ proc ::ftpd::command::STOR {sock list} {
         }
     }
 
-#
-# Patched Mark O'Connor
-#
+    #
+    # Patched Mark O'Connor
+    #
     if {![catch {::ftpd::Fs store $path $data(mode)} f]} {
 	puts $sock "150 Copy Started ($data(mode))"
 	fcopy $data(sock2) $f -command [list ::ftpd::GetDone $sock $data(sock2) $f ""]
@@ -1436,9 +1466,9 @@ proc ::ftpd::command::STOU {sock list} {
         incr i
     }
 
-#
-# Patched Mark O'Connor
-#
+    #
+    # Patched Mark O'Connor
+    #
     if {![catch {::ftpd::Fs store $file $data(mode)} f]} {
 	puts $sock "150 Copy Started ($data(mode))"
 	fcopy $data(sock2) $f -command [list ::ftpd::GetDone $sock $data(sock2) $f $file]
@@ -1568,6 +1598,8 @@ proc ::ftpd::command::USER {sock list} {
 
 proc ::ftpd::GetDone {sock sock2 f filename bytes {err {}}} {
     upvar #0 ::ftpd::$sock data
+    variable cfg
+
     close $f
     FinishData $sock
 
@@ -1578,6 +1610,9 @@ proc ::ftpd::GetDone {sock sock2 f filename bytes {err {}}} {
     } else {
         puts $sock "226 Transfer complete (unique file name: $filename)."
     }
+    if {[hasCallback xferDoneCmd]} then {
+	catch {$cfg(xferDoneCmd) $sock $sock2 $f $bytes $filename $err}
+    }
     Log debug "GetDone $f $sock2 $bytes bytes filename: $filename"
     return
 }
@@ -1619,7 +1654,7 @@ namespace eval ::ftpd::fsFile {
     # Our document root directory
 
     variable docRoot
-    if {![info exist docRoot]} {
+    if {![info exists docRoot]} {
 	set docRoot /
     }
 
@@ -1709,38 +1744,40 @@ proc ::ftpd::fsFile::fs {command path args} {
 
     switch -exact -- $command {
         append {
-          #
-          # Patched Mark O'Connor
-          #
+	    #
+	    # Patched Mark O'Connor
+	    #
 	    set fhandle [open $path a]
-          if {[lindex $args 0] == "binary"} {
-             fconfigure $fhandle -translation binary
-          }
-          return $fhandle
+	    if {[lindex $args 0] == "binary"} {
+		fconfigure $fhandle -translation binary -encoding binary
+	    }
+	    return $fhandle
         }
 	retr {
-          #
-          # Patched Mark O'Connor
-          #
+	    #
+	    # Patched Mark O'Connor
+	    #
 	    set fhandle [open $path r]
-          if {[lindex $args 0] == "binary"} {
-             fconfigure $fhandle -translation binary
-          }
-          return $fhandle
+	    if {[lindex $args 0] == "binary"} {
+		fconfigure $fhandle -translation binary -encoding binary
+	    }
+	    return $fhandle
 	}
 	store {
-          #
-          # Patched Mark O'Connor
-          #
+	    #
+	    # Patched Mark O'Connor
+	    #
 	    set fhandle [open $path w]
-          if {[lindex $args 0] == "binary"} {
-             fconfigure $fhandle -translation binary
-          }
-          return $fhandle
+	    if {[lindex $args 0] == "binary"} {
+		fconfigure $fhandle -translation binary -encoding binary
+	    }
+	    return $fhandle
 	}
 	dlist {
 	    foreach {style outchan} $args break
-	    set path [glob -nocomplain $path]
+	    ::ftpd::Log debug "at dlist {$style} {$outchan} {$path}"
+	    #set path [glob -nocomplain $path]
+	    #::ftpd::Log debug "at dlist2 {$style} {$outchan} {$path}"
 
             # Attempt to get a list of all files (even ones that start with .)
 
@@ -1756,19 +1793,31 @@ proc ::ftpd::fsFile::fs {command path args} {
 
             set fileList [lsort -unique [concat [glob -nocomplain $path1] \
                     [glob -nocomplain $path2]]]
+	    
+	    ::ftpd::Log debug "File list is {$fileList}"
 
 	    switch -- $style {
 	        nlst {
+		    ::ftpd::Log debug "In nlist"
 	            foreach f [lsort $fileList] {
                         if {[string equal [file tail $f] "."] || \
                                 [string equal [file tail $f] ".."]} {
                             continue
                         }
-		        puts $outchan $f
+			if {[string equal {} $f]} then continue
+			::ftpd::Log debug [file tail $f]
+		        puts $outchan [file tail $f]
 	            }
 	        }
 		list {
+		    # [ 766112 ] report . and .. directories (linux)
+		    # Copied the code from 'nlst' above to handle this.
+
 	            foreach f [lsort $fileList] {
+                        if {[string equal [file tail $f] "."] || \
+                                [string equal [file tail $f] ".."]} {
+                            continue
+                        }
 			file stat $f stat
                         if {[string equal $tcl_platform(platform) "unix"]} {
                             set user [file attributes $f -owner]
@@ -1895,7 +1944,7 @@ proc ::ftpd::fsFile::fs {command path args} {
 # Side Effects:
 #       None.
 
-proc ftpd::fsFile::PermBits {file mode} {
+proc ::ftpd::fsFile::PermBits {file mode} {
 
     array set s {
         0 --- 1 --x 2 -w- 3 -wx 4 r-- 5 r-x 6 rw- 7 rwx
@@ -1927,7 +1976,7 @@ proc ftpd::fsFile::PermBits {file mode} {
 # Side Effects:
 #       None.
 
-proc ftpd::fsFile::FormDate {seconds} {
+proc ::ftpd::fsFile::FormDate {seconds} {
 
     set currentTime [clock seconds]
     set oldTime [clock scan "6 months ago" -base $currentTime]
@@ -1947,5 +1996,50 @@ proc ftpd::fsFile::FormDate {seconds} {
 #
 # Patched Mark O'Connor
 #
-package provide ftpd 1.1.1
+package provide ftpd 1.2.1
+
+
+##
+## Implementation of passive command
+##
+proc ::ftpd::command::PASV {sock args} {
+    upvar #0 ::ftpd::$sock data
+
+    set data(sock2a) [socket -server [list ::ftpd::PasvAccept $sock] 0]
+    set list1 [fconfigure $sock -sockname]
+    set ip [lindex $list1 0]
+    set list2 [fconfigure $data(sock2a) -sockname]
+    set port [lindex $list2 2]
+    ::ftpd::Log debug "PASV on {$list1} {$list2} $ip $port"
+    set ans [split $ip {.}]
+    lappend ans [expr {($port >> 8) & 0xff}] [expr {$port & 0xff}]
+    set ans [join $ans {,}]
+    puts $sock "227 Entering Passive Mode ($ans)."
+    return
+}
+
+
+proc ::ftpd::PasvAccept {sock sock2 ip port} {
+    upvar #0 ::ftpd::$sock data
+
+    ::ftpd::Log debug "In Pasv Accept with {$sock} {$sock2} {$ip} {$port}"
+    ##
+    ## Verify this is from who it should be
+    ##
+    if {![string equal $ip $data(ip)]} then {
+	##
+	## Nope, so close it and wait some more
+	##
+	close $sock2
+	return
+    }
+    ::ftpd::FinishData $sock
+
+    set data(sock2) $sock2
+    fconfigure $data(sock2) -translation $data(mode)
+    close $data(sock2a)
+    set data(sock2a) ""
+    return
+}
+
 
