@@ -23,9 +23,13 @@
 # repeated.  The names and values are encoded, and this module takes care
 # of decoding them.
 
-package provide ncgi 1.1
+# We use newer string routines
+package require Tcl 8.2
+package require fileutil ; # Required by importFile.
 
-namespace eval ncgi {
+package provide ncgi 1.2.3
+
+namespace eval ::ncgi {
 
     # "query" holds the raw query (i.e., form) data
     # This is treated as a cache, too, so you can call ncgi::query more than
@@ -81,14 +85,23 @@ namespace eval ncgi {
     array set map {
 	" " +   \n %0D%0A
     }
- 
+
+    # Map of transient files
+
+    variable  _tmpfiles
+    array set _tmpfiles {}
+
     # I don't like importing, but this makes everything show up in 
     # pkgIndex.tcl
 
-    namespace export *
+    namespace export reset urlStub query type decode encode
+    namespace export nvlist parse input value valueList
+    namespace export setValue setValueList setDefaultValue setDefaultValueList
+    namespace export empty import importAll importFile redirect header
+    namespace export parseMimeValue multipart cookie setCookie
 }
 
-# ncgi::reset
+# ::ncgi::reset
 #
 #	This resets the state of the CGI input processor.  This is primarily
 #	used for tests, although it is also designed so that TclHttpd can
@@ -107,22 +120,26 @@ namespace eval ncgi {
 #	Resets the cached query data and wipes any environment variables
 #	associated with CGI inputs (like QUERY_STRING)
 
-proc ncgi::reset {args} {
+proc ::ncgi::reset {args} {
     global env
+    variable _tmpfiles
     variable query
     variable contenttype
     variable cookieOutput
 
+    # array unset _tmpfiles -- Not a Tcl 8.2 idiom
+    unset _tmpfiles ; array set _tmpfiles {}
+
     set cookieOutput {}
     if {[llength $args] == 0} {
 
 	# We use and test args here so we can detect the
 	# difference between empty query data and a full reset.
 
-	if {[info exist query]} {
+	if {[info exists query]} {
 	    unset query
 	}
-	if {[info exist contenttype]} {
+	if {[info exists contenttype]} {
 	    unset contenttype
 	}
     } else {
@@ -131,7 +148,7 @@ proc ncgi::reset {args} {
     }
 }
 
-# ncgi::urlStub
+# ::ncgi::urlStub
 #
 #	Set or return the URL associated with the current page.
 #	This is for use by TclHttpd to override the default value
@@ -144,14 +161,15 @@ proc ncgi::reset {args} {
 # Side Effects:
 #	May affects future calls to ncgi::urlStub
 
-proc ncgi::urlStub {{url {}}} {
+proc ::ncgi::urlStub {{url {}}} {
+    global   env
     variable urlStub
     if {[string length $url]} {
 	set urlStub $url
 	return ""
-    } elseif {[info exist urlStub]} {
+    } elseif {[info exists urlStub]} {
 	return $urlStub
-    } elseif {[info exist env(SCRIPT_NAME)]} {
+    } elseif {[info exists env(SCRIPT_NAME)]} {
 	set urlStub $env(SCRIPT_NAME)
 	return $urlStub
     } else {
@@ -159,7 +177,7 @@ proc ncgi::urlStub {{url {}}} {
     }
 }
 
-# ncgi::query
+# ::ncgi::query
 #
 #	This reads the query data from the appropriate location, which depends
 #	on if it is a POST or GET request.
@@ -170,18 +188,18 @@ proc ncgi::urlStub {{url {}}} {
 # Results:
 #	The raw query data.
 
-proc ncgi::query {} {
+proc ::ncgi::query {} {
     global env
     variable query
 
-    if {[info exist query]} {
+    if {[info exists query]} {
 	# This ensures you can call ncgi::query more than once,
 	# and that you can use it with ncgi::reset
 	return $query
     }
 
     set query ""
-    if {[info exist env(REQUEST_METHOD)]} {
+    if {[info exists env(REQUEST_METHOD)]} {
 	if {$env(REQUEST_METHOD) == "GET"} {
 	    if {[info exists env(QUERY_STRING)]} {
 		set query $env(QUERY_STRING)
@@ -189,6 +207,8 @@ proc ncgi::query {} {
 	} elseif {$env(REQUEST_METHOD) == "POST"} {
 	    if {[info exists env(CONTENT_LENGTH)] &&
 		    [string length $env(CONTENT_LENGTH)] != 0} {
+ 		## added by Steve Cassidy to try to fix binary file upload
+ 		fconfigure stdin -translation binary -encoding binary
 		set query [read stdin $env(CONTENT_LENGTH)]
 	    }
 	}
@@ -196,7 +216,7 @@ proc ncgi::query {} {
     return $query
 }
 
-# ncgi::type
+# ::ncgi::type
 #
 #	This returns the content type of the query data.
 #
@@ -206,12 +226,12 @@ proc ncgi::query {} {
 # Results:
 #	The content type of the query data.
 
-proc ncgi::type {} {
+proc ::ncgi::type {} {
     global env
     variable contenttype
 
-    if {![info exist contenttype]} {
-	if {[info exist env(CONTENT_TYPE)]} {
+    if {![info exists contenttype]} {
+	if {[info exists env(CONTENT_TYPE)]} {
 	    set contenttype $env(CONTENT_TYPE)
 	} else {
 	    return ""
@@ -220,7 +240,7 @@ proc ncgi::type {} {
     return $contenttype
 }
 
-# ncgi::decode
+# ::ncgi::decode
 #
 #	This decodes data in www-url-encoded format.
 #
@@ -230,21 +250,19 @@ proc ncgi::type {} {
 # Results:
 #	The decoded value
 
-proc ncgi::decode {str} {
-    # Replace + with space
-    regsub -all -- {\+} $str { } str
+proc ::ncgi::decode {str} {
+    # rewrite "+" back to space
+    # protect \ from quoting another '\'
+    set str [string map [list + { } "\\" "\\\\"] $str]
 
-    # Protect Tcl special chars
-    regsub -all -- {[][\\\$]} $str {\\&} str
+    # prepare to process all %-escapes
+    regsub -all -- {%([A-Fa-f0-9][A-Fa-f0-9])} $str {\\u00\1} str
 
-    # Replace %xx sequences with a format command
-    regsub -all -- {%([0-9a-fA-F][0-9a-fA-F])} $str {[format %c 0x\1]} str
-
-    # Replace the format commands with their result
-    return [subst $str]
+    # process \u unicode mapped chars
+    return [subst -novar -nocommand $str]
 }
 
-# ncgi::encode
+# ::ncgi::encode
 #
 #	This encodes data in www-url-encoded format.
 #
@@ -254,7 +272,7 @@ proc ncgi::decode {str} {
 # Results:
 #	The encoded value
 
-proc ncgi::encode {string} {
+proc ::ncgi::encode {string} {
     variable map
 
     # 1 leave alphanumerics characters alone
@@ -263,18 +281,13 @@ proc ncgi::encode {string} {
     # 4 "subst" the result, doing all the array substitutions
 
     regsub -all -- \[^a-zA-Z0-9\] $string {$map(&)} string
-    if {0} {
-	# These lines seem totally bogus - am I missing something?
-	regsub -all -- \n $string {\\n} string
-	regsub -all -- \t $string {\\t} string
-    }
-    # This quotes cases like $map([) or $map($)
+    # This quotes cases like $map([) or $map($) => $map(\[) ...
     regsub -all -- {[][{})\\]\)} $string {\\&} string
-    return [subst $string]
+    return [subst -nocommand $string]
 }
 
 
-# ncgi::nvlist
+# ::ncgi::nvlist
 #
 #	This parses the query data and returns it as a name, value list
 #
@@ -287,9 +300,9 @@ proc ncgi::encode {string} {
 # Results:
 #	An alternating list of names and values
 
-proc ncgi::nvlist {} {
-    set query [ncgi::query]
-    set type [ncgi::type]
+proc ::ncgi::nvlist {} {
+    set query [query]
+    set type  [type]
     switch -glob -- $type {
 	"" -
 	application/x-www-form-urlencoded -
@@ -308,12 +321,12 @@ proc ncgi::nvlist {} {
 		    set varname anonymous
 		    set val $x
 		}
-		lappend result [ncgi::decode $varname] [ncgi::decode $val]
+		lappend result [decode $varname] [decode $val]
 	    }
 	    return $result
 	}
 	multipart/* {
-	    return [ncgi::multipart $type $query]
+	    return [multipart $type $query]
 	}
 	default {
 	    return -code error "Unknown Content-Type: $type"
@@ -321,7 +334,7 @@ proc ncgi::nvlist {} {
     }
 }
 
-# ncgi::parse
+# ::ncgi::parse
 #
 #	The parses the query data and stores it into an array for later retrieval.
 #	You should use the ncgi::value or ncgi::valueList procedures to get those
@@ -336,15 +349,15 @@ proc ncgi::nvlist {} {
 # Results:
 #	A list of names of the query values
 
-proc ncgi::parse {} {
+proc ::ncgi::parse {} {
     variable value
     variable listRestrict 0
     variable varlist {}
-    if {[info exist value]} {
+    if {[info exists value]} {
 	unset value
     }
-    foreach {name val} [ncgi::nvlist] {
-	if {![info exist value($name)]} {
+    foreach {name val} [nvlist] {
+	if {![info exists value($name)]} {
 	    lappend varlist $name
 	}
 	lappend value($name) $val
@@ -352,7 +365,7 @@ proc ncgi::parse {} {
     return $varlist
 } 
 
-# ncgi::input
+# ::ncgi::input
 #
 #	Like ncgi::parse, but with Don Libes cgi.tcl semantics.
 #	Form elements must have a trailing "List" in their name to be
@@ -365,22 +378,22 @@ proc ncgi::parse {} {
 # Results:
 #	The list of element names in the form
 
-proc ncgi::input {{fakeinput {}} {fakecookie {}}} {
+proc ::ncgi::input {{fakeinput {}} {fakecookie {}}} {
     variable value
     variable varlist {}
     variable listRestrict 1
-    if {[info exist value]} {
+    if {[info exists value]} {
 	unset value
     }
     if {[string length $fakeinput]} {
 	ncgi::reset $fakeinput
     }
-    foreach {name val} [ncgi::nvlist] {
-	set exists [info exist value($name)]
+    foreach {name val} [nvlist] {
+	set exists [info exists value($name)]
 	if {!$exists} {
 	    lappend varlist $name
 	}
-	if {[regexp -- List$ $name]} {
+	if {[string match "*List" $name]} {
 	    # Accumulate a list of values for this name
 	    lappend value($name) $val
 	} elseif {$exists} {
@@ -395,7 +408,7 @@ proc ncgi::input {{fakeinput {}} {fakecookie {}}} {
     return $varlist
 } 
 
-# ncgi::value
+# ::ncgi::value
 #
 #	Return the value of a named query element, or the empty string if
 #	it was not not specified.  This only returns the first value of
@@ -409,14 +422,14 @@ proc ncgi::input {{fakeinput {}} {fakecookie {}}} {
 # Results:
 #	The first value of the named element, or the default
 
-proc ncgi::value {key {default {}}} {
+proc ::ncgi::value {key {default {}}} {
     variable value
     variable listRestrict
     variable contenttype
     if {[info exists value($key)]} {
 	if {$listRestrict} {
-	    
-	    # ncgi::input was called, and it already figured out if the
+
+	    # ::ncgi::input was called, and it already figured out if the
 	    # user wants list structure or not.
 
 	    set val $value($key)
@@ -426,8 +439,8 @@ proc ncgi::value {key {default {}}} {
 
 	    set val [lindex $value($key) 0]
 	}
-	if {[string match multipart/* [ncgi::type]]} {
-	    
+	if {[string match multipart/* [type]]} {
+
 	    # Drop the meta-data information associated with each part
 
 	    set val [lindex $val 1]
@@ -438,7 +451,7 @@ proc ncgi::value {key {default {}}} {
     }
 }
 
-# ncgi::valueList
+# ::ncgi::valueList
 #
 #	Return all the values of a named query element as a list, or
 #	the empty list if it was not not specified.  This always returns
@@ -451,7 +464,7 @@ proc ncgi::value {key {default {}}} {
 # Results:
 #	The first value of the named element, or ""
 
-proc ncgi::valueList {key {default {}}} {
+proc ::ncgi::valueList {key {default {}}} {
     variable value
     if {[info exists value($key)]} {
 	return $value($key)
@@ -460,7 +473,7 @@ proc ncgi::valueList {key {default {}}} {
     }
 }
 
-# ncgi::setValue
+# ::ncgi::setValue
 #
 #	Jam a new value into the CGI environment.  This is handy for preliminary
 #	processing that does data validation and cleanup.
@@ -475,7 +488,7 @@ proc ncgi::valueList {key {default {}}} {
 # Side Effects:
 #	Alters the ncgi::value and possibly the ncgi::valueList variables
 
-proc ncgi::setValue {key value} {
+proc ::ncgi::setValue {key value} {
     variable listRestrict
     if {$listRestrict} {
 	ncgi::setValueList $key $value
@@ -484,7 +497,7 @@ proc ncgi::setValue {key value} {
     }
 }
 
-# ncgi::setValueList
+# ::ncgi::setValueList
 #
 #	Jam a list of new values into the CGI environment.
 #
@@ -496,17 +509,25 @@ proc ncgi::setValue {key value} {
 # Side Effects:
 #	Alters the ncgi::value and possibly the ncgi::valueList variables
 
-proc ncgi::setValueList {key valuelist} {
+proc ::ncgi::setValueList {key valuelist} {
     variable value
     variable varlist
-    if {![info exist value($key)]} {
+    if {![info exists value($key)]} {
 	lappend varlist $key
     }
-    set value($key) $valuelist
+
+    # This if statement is a workaround for another hack in
+    # ::ncgi::value that treats multipart form data
+    # differently.
+    if {[string match multipart/* [type]]} {
+	set value($key) [list [list {} [join $valuelist]]]
+    } else {
+	set value($key) $valuelist
+    }
     return ""
 }
 
-# ncgi::setDefaultValue
+# ::ncgi::setDefaultValue
 #
 #	Set a new value into the CGI environment if there is not already one there.
 #
@@ -519,11 +540,11 @@ proc ncgi::setValueList {key valuelist} {
 # Side Effects:
 #	Alters the ncgi::value and possibly the ncgi::valueList variables
 
-proc ncgi::setDefaultValue {key value} {
+proc ::ncgi::setDefaultValue {key value} {
     ncgi::setDefaultValueList $key [list $value]
 }
 
-# ncgi::setDefaultValueList
+# ::ncgi::setDefaultValueList
 #
 #	Jam a list of new values into the CGI environment if the CGI value
 #	is not already defined.
@@ -536,9 +557,9 @@ proc ncgi::setDefaultValue {key value} {
 # Side Effects:
 #	Alters the ncgi::value and possibly the ncgi::valueList variables
 
-proc ncgi::setDefaultValueList {key valuelist} {
+proc ::ncgi::setDefaultValueList {key valuelist} {
     variable value
-    if {![info exist value($key)]} {
+    if {![info exists value($key)]} {
 	ncgi::setValueList $key $valuelist
 	return ""
     } else {
@@ -546,7 +567,7 @@ proc ncgi::setDefaultValueList {key valuelist} {
     }
 }
 
-# ncgi::empty --
+# ::ncgi::empty --
 #
 #	Return true if the CGI variable doesn't exist or is an empty string
 #
@@ -556,11 +577,11 @@ proc ncgi::setDefaultValueList {key valuelist} {
 # Results:
 #	1 if the variable doesn't exist or has the empty value
 
-proc ncgi::empty {name} {
-    return [expr {[string length [string trim [ncgi::value $name]]] == 0}]
+proc ::ncgi::empty {name} {
+    return [expr {[string length [string trim [value $name]]] == 0}]
 }
 
-# ncgi::import
+# ::ncgi::import
 #
 #	Map a CGI input into a Tcl variable.  This creates a Tcl variable in
 #	the callers scope that has the value of the CGI input.  An alternate
@@ -571,16 +592,16 @@ proc ncgi::empty {name} {
 #	tclname		If present, an alternate name for the Tcl variable,
 #			otherwise it is the same as the form element name
 
-proc ncgi::import {cginame {tclname {}}} {
+proc ::ncgi::import {cginame {tclname {}}} {
     if {[string length $tclname]} {
 	upvar 1 $tclname var
     } else {
 	upvar 1 $cginame var
     }
-    set var [ncgi::value $cginame]
+    set var [value $cginame]
 }
 
-# ncgi::importAll
+# ::ncgi::importAll
 #
 #	Map a CGI input into a Tcl variable.  This creates a Tcl variable in
 #	the callers scope for every CGI value, or just for those named values.
@@ -589,18 +610,18 @@ proc ncgi::import {cginame {tclname {}}} {
 #	args	A list of form element names.  If this is empty,
 #		then all form value are imported.
 
-proc ncgi::importAll {args} {
+proc ::ncgi::importAll {args} {
     variable varlist
     if {[llength $args] == 0} {
 	set args $varlist
     }
     foreach cginame $args {
 	upvar 1 $cginame var
-	set var [ncgi::value $cginame]
+	set var [value $cginame]
     }
 }
 
-# ncgi::redirect
+# ::ncgi::redirect
 #
 #	Generate a redirect by returning a header that has a Location: field.
 #	If the URL is not absolute, this automatically qualifies it to
@@ -612,7 +633,7 @@ proc ncgi::importAll {args} {
 # Side Effects:
 #	Outputs a redirect header
 
-proc ncgi::redirect {url} {
+proc ::ncgi::redirect {url} {
     global env
 
     if {![regexp -- {^[^:]+://} $url]} {
@@ -626,17 +647,17 @@ proc ncgi::redirect {url} {
 	#		current one in base Basic Authentication is being used.
 	# port		This is set if it is not the default port.
 
-	if {[info exist env(REQUEST_URI)]} {
+	if {[info exists env(REQUEST_URI)]} {
 	    # Not all servers have the leading protocol spec
 	    regsub -- {^https?://[^/]*/} $env(REQUEST_URI) / request_uri
-	} elseif {[info exist env(SCRIPT_NAME)]} {
+	} elseif {[info exists env(SCRIPT_NAME)]} {
 	    set request_uri $env(SCRIPT_NAME)
 	} else {
 	    set request_uri /
 	}
 
 	set port ""
-	if {[info exist env(HTTPS)] && $env(HTTPS) == "on"} {
+	if {[info exists env(HTTPS)] && $env(HTTPS) == "on"} {
 	    set proto https
 	    if {$env(SERVER_PORT) != 443} {
 		set port :$env(SERVER_PORT)
@@ -651,7 +672,12 @@ proc ncgi::redirect {url} {
 	# URL.  Otherwise use SERVER_NAME.  These could be different, e.g.,
 	# "pop.scriptics.com" vs. "pop"
 
-	if {![regexp -- {^https?://([^/:]*)} $env(REQUEST_URI) x server]} {
+	if {[info exists env(REQUEST_URI)]} {
+	    # Not all servers have the leading protocol spec
+	    if {![regexp -- {^https?://([^/:]*)} $env(REQUEST_URI) x server]} {
+		set server $env(SERVER_NAME)
+	    }
+	} else {
 	    set server $env(SERVER_NAME)
 	}
 	if {[string match /* $url]} {
@@ -676,13 +702,13 @@ proc ncgi::redirect {url} {
 # Side Effects:
 #	Outputs a normal header
 
-proc ncgi::header {{type text/html} args} {
+proc ::ncgi::header {{type text/html} args} {
     variable cookieOutput
     puts "Content-Type: $type"
     foreach {n v} $args {
 	puts "$n: $v"
     }
-    if {[info exist cookieOutput]} {
+    if {[info exists cookieOutput]} {
 	foreach line $cookieOutput {
 	    puts "Set-Cookie: $line"
 	}
@@ -691,7 +717,7 @@ proc ncgi::header {{type text/html} args} {
     flush stdout
 }
 
-# ncgi::parseMimeValue
+# ::ncgi::parseMimeValue
 #
 #	Parse a MIME header value, which has the form
 #	value; param=value; param2="value2"; param3='value3'
@@ -709,7 +735,7 @@ proc ncgi::header {{type text/html} args} {
 #		{param value param2 value param3 value3}
 #	}
 
-proc ncgi::parseMimeValue {value} {
+proc ::ncgi::parseMimeValue {value} {
     set parts [split $value \;]
     set results [list [string trim [lindex $parts 0]]]
     set paramList [list]
@@ -733,7 +759,7 @@ proc ncgi::parseMimeValue {value} {
     return $results
 }
 
-# ncgi::multipart
+# ::ncgi::multipart
 #
 #	This parses multipart form data.
 #	Based on work by Steve Ball for TclHttpd, but re-written to use
@@ -775,9 +801,9 @@ proc ncgi::parseMimeValue {value} {
 #	example above.  Finally, not that if the value has a second element,
 #	which are the parameters, you can "array set" that as well.
 #	
-proc ncgi::multipart {type query} {
+proc ::ncgi::multipart {type query} {
 
-    set parsedType [ncgi::parseMimeValue $type]
+    set parsedType [parseMimeValue $type]
     if {![string match multipart/* [lindex $parsedType 0]]} {
 	return -code error "Not a multipart Content-Type: [lindex $parsedType 0]"
     }
@@ -788,15 +814,15 @@ proc ncgi::multipart {type query} {
     set boundary $options(boundary)
 
     # The query data is typically read in binary mode, which preserves
-    # the \r\n sequence from a Windows-based browser.  
+    # the \r\n sequence from a Windows-based browser.
     # Also, binary data may contain \r\n sequences.
 
-    if {[regexp -- $boundary\r\n $query]} {
+    if {[string match "*$boundary\r\n*" $query]} {
         set lineDelim "\r\n"
-puts "DELIM"
+	#	puts "DELIM"
     } else {
         set lineDelim "\n"
-puts "NO"
+	#	puts "NO"
     }
 
     # Iterate over the boundary string and chop into parts
@@ -876,7 +902,72 @@ puts "NO"
     return $results
 }
 
-# ncgi::cookie
+# ::ncgi::importFile --
+#
+#   get information about a file upload field
+#
+# Arguments:
+#   cmd         one of '-server' '-client' '-type' '-data'
+#   var         cgi variable name for the file field
+#   filename    filename to write to for -server
+# Results:
+#   -server returns the name of the file on the server: side effect
+#      is that the file gets stored on the server and the 
+#      script is responsible for deleting/moving the file
+#   -client returns the name of the file sent from the client 
+#   -type   returns the mime type of the file
+#   -data   returns the contents of the file 
+
+proc ::ncgi::importFile {cmd var {filename {}}} {
+
+    set vlist [valueList $var]
+
+    array set fileinfo [lindex [lindex $vlist 0] 0]
+    set contents [lindex [lindex $vlist 0] 1]
+
+    switch -exact -- $cmd {
+	-server {
+	    ## take care not to write it out more than once
+	    variable _tmpfiles
+	    if {![info exists _tmpfiles($var)]} {
+		if {$filename != {}} {
+		    ## use supplied filename 
+		    set _tmpfiles($var) $filename
+		} else {
+		    ## create a tmp file 
+		    set _tmpfiles($var) [::fileutil::tempfile ncgi]
+		}
+
+		# write out the data only if it's not been done already
+		if {[catch {open $_tmpfiles($var) w} h]} {
+		    error "Can't open temporary file in ncgi::importFile ($h)"
+		} 
+
+		fconfigure $h -translation binary -encoding binary
+		puts -nonewline $h $contents 
+		close $h
+	    }
+	    return $_tmpfiles($var)
+	}
+	-client {
+	    if {![info exists fileinfo(filename)]} {return {}}
+	    return $fileinfo(filename)
+	}
+	-type {
+	    if {![info exists fileinfo(content-type)]} {return {}}
+	    return $fileinfo(content-type)
+	}
+	-data {
+	    return $contents
+	}
+	default {
+	    error "Unknown subcommand to ncgi::import_file: $cmd"
+	}
+    }
+}
+
+
+# ::ncgi::cookie
 #
 #	Return a *list* of cookie values, if present, else ""
 #	It is possible for multiple cookies with the same key
@@ -888,10 +979,10 @@ puts "NO"
 # Results:
 #	A list of values for the cookie
 
-proc ncgi::cookie {cookie} {
+proc ::ncgi::cookie {cookie} {
     global env
     set result ""
-    if {[info exist env(HTTP_COOKIE)]} {
+    if {[info exists env(HTTP_COOKIE)]} {
 	foreach pair [split $env(HTTP_COOKIE) \;] {
 	    foreach {key value} [split [string trim $pair] =] { break ;# lassign }
 	    if {[string compare $cookie $key] == 0} {
@@ -902,7 +993,7 @@ proc ncgi::cookie {cookie} {
     return $result
 }
 
-# ncgi::setCookie
+# ::ncgi::setCookie
 #
 #	Set a return cookie.  You must call this before you call
 #	ncgi::header or ncgi::redirect
@@ -918,16 +1009,16 @@ proc ncgi::cookie {cookie} {
 # Side Effects:
 #	Formats and stores the Set-Cookie header for the reply.
 
-proc ncgi::setCookie {args} {
+proc ::ncgi::setCookie {args} {
     variable cookieOutput
     array set opt $args
     set line "$opt(-name)=$opt(-value) ;"
     foreach extra {path domain} {
-	if {[info exist opt(-$extra)]} {
+	if {[info exists opt(-$extra)]} {
 	    append line " $extra=$opt(-$extra) ;"
 	}
     }
-    if {[info exist opt(-expires)]} {
+    if {[info exists opt(-expires)]} {
 	switch -glob -- $opt(-expires) {
 	    *GMT {
 		set expires $opt(-expires)
@@ -939,7 +1030,7 @@ proc ncgi::setCookie {args} {
 	}
 	append line " expires=$expires ;"
     }
-    if {[info exist opt(-secure)]} {
+    if {[info exists opt(-secure)]} {
 	append line " secure "
     }
     lappend cookieOutput $line
