@@ -2,15 +2,15 @@
 #
 #	Implementation of report objects for Tcl.
 #
-# Copyright (c) 2001 by Andreas Kupries <a.kupries@westend.com>
+# Copyright (c) 2001 by Andreas Kupries <andreas_kupries@users.sourceforge.net>
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id: report.tcl,v 1.5 2001/10/17 17:27:26 andreas_kupries Exp $
+# RCS: @(#) $Id$
 
 package require Tcl 8.2
-package provide report 0.2
+package provide report 0.3.1
 
 namespace eval ::report {
     # Data storage in the report module
@@ -1354,6 +1354,7 @@ proc ::report::FormatCell {value size just} {
 	    if {$vlen < $size} {
 		return [string repeat " " [expr {$size - $vlen}]]$value
 	    }
+	    incr size -1
 	    return [string range $value 0 $size]
 	}
 	center {
