@@ -141,6 +141,7 @@ proc ::textutil::expander::Methods {name method argList} {
         evalcmd -
 	textcmd -
         cpush -
+	ctopandclear -
         cis -
         cname -
         cset -
@@ -355,7 +356,6 @@ proc ::textutil::expander::Op_errmode {name {newErrmode ""}} {
 
 proc ::textutil::expander::Op_evalcmd {name {newEvalCmd ""}} {
     if {[string length $newEvalCmd] != 0} {
-
         Set evalcmd $newEvalCmd
     }
     return [Get evalcmd]
@@ -403,7 +403,9 @@ proc ::textutil::expander::Op_reset {name} {
     variable Info 
 
     if {[info exists Info($name-lb)]} {
-        array unset Info "$name-*"
+        foreach elt [array names Info "$name-*"] {
+            unset Info($elt)
+        }
     }
 
     set Info($name-lb) "\["
@@ -440,8 +442,11 @@ proc ::textutil::expander::Op_reset {name} {
 #       will be added to this context until it is popped.
 
 proc ::textutil::expander::Op_cpush {name cname} {
+    # FRINK: nocheck
     incr [Var level]
+    # FRINK: nocheck
     set [Var output-[Get level]] {}
+    # FRINK: nocheck
     set [Var name-[Get level]] $cname
 }
 
@@ -460,7 +465,7 @@ proc ::textutil::expander::Op_cpush {name cname} {
 #	false otherwise.
 
 proc ::textutil::expander::Op_cis {name cname} {
-    return [expr [string compare $cname [Op_cname $name]] == 0]
+    return [expr {[string compare $cname [Op_cname $name]] == 0}]
 }
 
 #---------------------------------------------------------------------
@@ -514,9 +519,8 @@ proc ::textutil::expander::Op_cset {name varname value} {
 
 proc ::textutil::expander::Op_cget {name varname} {
     if {![info exists [Var data-[Get level]-$varname]]} {
-        error "$name cget: $varname doesn't exist in this context"
+        error "$name cget: $varname doesn't exist in this context ([Get level])"
     }
-
     return [Get data-[Get level]-$varname]
 }
 
@@ -569,13 +573,45 @@ proc ::textutil::expander::Op_cpop {name cname} {
     }
 
     set result [Get output-[Get level]]
+    # FRINK: nocheck
     set [Var output-[Get level]] ""
+    # FRINK: nocheck
     set [Var name-[Get level]] ""
 
-    array unset "Info data-[Get level]-*"
+    foreach elt [array names "Info data-[Get level]-*"] {
+        unset Info($elt)
+    }
 
+    # FRINK: nocheck
     incr [Var level] -1
+    return $result
+}
 
+#---------------------------------------------------------------------
+# FUNCTION:
+# 	Op_ctopandclear
+#
+# INPUTS:
+#	None.
+#
+# RETURNS:
+#	The accumulated output in the topmost context, clears the context,
+#	but does not pop it.
+#
+# DESCRIPTION:
+#       Returns the accumulated output for the current context, first
+#	popping the context from the stack.  The expected context name
+#	must match the real name, or an error occurs.
+
+proc ::textutil::expander::Op_ctopandclear {name} {
+    variable Info
+
+    if {[Get level] == 0} {
+        error "$name cpop underflow on '[Op_cname $name]'"
+    }
+
+    set result [Get output-[Get level]]
+    Set output-[Get level] ""
     return $result
 }
 
@@ -593,6 +629,7 @@ proc ::textutil::expander::Op_cpop {name cname} {
 #       Appends the text to the accumulated output in the current context.
 
 proc ::textutil::expander::Op_cappend {name text} {
+    # FRINK: nocheck
     append [Var output-[Get level]] $text
 }
 
@@ -619,12 +656,17 @@ proc ::textutil::expander::Op_cappend {name text} {
 #	brackets are used.
 
 proc ::textutil::expander::Op_expand {name inputString {brackets ""}} {
+
     # FIRST, push a new context onto the stack, and save the current
     # brackets.
+
     Op_cpush $name expand
     Op_cset $name lb [Get lb]
     Op_cset $name rb [Get rb]
 
+    # SF Tcllib Bug #530056.
+    set start_level [Get level] ; # remember this for check at end
+
     # NEXT, use the user's brackets, if given.
     if {[llength $brackets] == 2} {
         Set lb [lindex $brackets 0]
@@ -655,8 +697,15 @@ proc ::textutil::expander::Op_expand {name inputString {brackets ""}} {
         }
 
         # NEXT, A macro is the next thing; process it.
-        if {[catch "GetMacro inputString" macro]} {
-            error "Error reading macro: $macro"
+        if {[catch {GetMacro inputString} macro]} {
+	    # SF tcllib bug 781973 ... Do not throw a regular
+	    # error. Use HandleError to give the user control of the
+	    # situation, via the defined error mode. The continue
+	    # intercepts if the user allows the expansion to run on,
+	    # yet we must not try to run the non-existing macro.
+
+	    HandleError $name macro $inputString $macro
+	    continue
         }
 
         # Expand the macro, and output the result, or
@@ -668,7 +717,39 @@ proc ::textutil::expander::Op_expand {name inputString {brackets ""}} {
 
 	HandleError $name macro $macro $result
     }
-    
+
+    # SF Tcllib Bug #530056.
+    if {[Get level] > $start_level} {
+	# The user macros pushed additional contexts, but forgot to
+	# pop them all. The main work here is to place all the still
+	# open contexts into the error message, and to produce
+	# syntactically correct english.
+
+	set c [list]
+	set n [expr {[Get level] - $start_level}]
+	if {$n == 1} {
+	    set ctx  context
+	    set verb was
+	} else {
+	    set ctx  contexts
+	    set verb were
+	}
+	for {incr n -1} {$n >= 0} {incr n -1} {
+	    lappend c [Get name-[expr {[Get level]-$n}]]
+	}
+	return -code error \
+		"The following $ctx pushed by the macros $verb not popped: [join $c ,]."
+    } elseif {[Get level] < $start_level} {
+	set n [expr {$start_level - [Get level]}]
+	if {$n == 1} {
+	    set ctx  context
+	} else {
+	    set ctx  contexts
+	}
+	return -code error \
+		"The macros popped $n more $ctx than they had pushed."
+    }
+
     Op_lb $name [Op_cget $name lb]
     Op_rb $name [Op_cget $name rb]
 
@@ -738,7 +819,7 @@ proc ::textutil::expander::ExtractToToken {string token mode} {
 
     # First, determine the offset
     switch $mode {
-        include { set offset [expr [string length $token] - 1] }
+        include { set offset [expr {[string length $token] - 1}] }
         exclude { set offset -1 }
         default { error "::expander::ExtractToToken: unknown mode $mode" }
     }
@@ -752,9 +833,9 @@ proc ::textutil::expander::ExtractToToken {string token mode} {
         set theText $theString
         set theString ""
     } else {
-        set newEnd [expr $tokenPos + $offset]
-        set newBegin [expr $newEnd + 1]
-        set theText [string range $theString 0 $newEnd]
+        set newEnd    [expr {$tokenPos + $offset}]
+        set newBegin  [expr {$newEnd + 1}]
+        set theText   [string range $theString 0 $newEnd]
         set theString [string range $theString $newBegin end]
     }
 
@@ -814,7 +895,7 @@ proc ::textutil::expander::StripBrackets {macro} {
     set rlen [string length [Get rb]]
     set tlen [string length $macro]
 
-    return [string range $macro $llen [expr $tlen - $rlen - 1]]
+    return [string range $macro $llen [expr {$tlen - $rlen - 1}]]
 }
 
 # Return 1 if the macro is properly bracketed, and 0 otherwise.
@@ -823,8 +904,8 @@ proc ::textutil::expander::IsBracketed {macro} {
     set rlen [string length [Get rb]]
     set tlen [string length $macro]
 
-    set leftEnd [string range $macro 0 [expr $llen - 1]]
-    set rightEnd [string range $macro [expr $tlen - $rlen] end]
+    set leftEnd  [string range $macro 0       [expr {$llen - 1}]]
+    set rightEnd [string range $macro [expr {$tlen - $rlen}] end]
 
     if {$leftEnd != [Get lb]} {
         return 0
@@ -838,4 +919,4 @@ proc ::textutil::expander::IsBracketed {macro} {
 # Provide the package only if the code above was read and executed
 # without error.
 
-package provide textutil::expander 1.0
+package provide textutil::expander 1.2.1
